/* collector1xn.c */
/*       10        20        30        40        50        60        70       */
/*3456789012345678901234567890123456789012345678901234567890123456789012345671*/
/*----------------------------------------------------------------------------*/



#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "global.h" 

/*-------------------1xn MODEL -----------------------------COLLECTOR-------*/
int collector(unit,par,in_u,in_nr,out_o,out_n,deriv_o,deriv_n,h,init_energy,
              qp_sum,tsim,init_flag,i_dead_nr,Q,point_dead,min10_flag,
              day_of_year,un_max,lenght,latitude,iam)
           int    unit;      /* current unit number */
           double **par;     /* parameter array par[unit][1..PARMAX]  */
           int    **in_u;    /* input unit array, input connections   */
           int    **in_nr;   /* input number array                    */
           double **out_o;   /* output array with privious state      */
           double **out_n;   /* output array with current state       */
           double **deriv_o; /* derivatives: used for diffential equ. */
           double **deriv_n; /*  new state of deriv                   */
           double h;         /* sim time step                         */
           double **init_energy; /* initial energy in RC-comp. [Ws]   */
           double ***qp_sum; /* energy to one node qp[unit][n][m]     */
           double tsim;      /* simulation time in [s] of this day    */
           int    init_flag; /* set at initial loop of sim            */
           int *i_dead_nr;   /* deadtime_component (history)          */
           double **Q;       /* enthalpie array                       */
           int **point_dead; /* deadtime component                    */
           int min10_flag;   /* each 10 minutes intervall set         */
           int day_of_year;  /* day_of_year                           */
           int un_max;       /* maximum number of units               */
           double lenght;    /* geographic lenght                     */
           double latitude;  /* geographic latitude                   */
           double **iam;     /* incidence angle modifier array        */

{
/*  etc/xfig.awk   
  
     TYPE 01
     PRIORITY  6          used by config routine
     HYDRAULIC 1          used by config routine, hydraulic 2 pole

    
     PAR1: 4.94 [m2] total absorber area
     PAR2: 0.83 [] eta0*F'
     PAR3: 0.6 [] eta0_diff*F'  (iam-reduced absorption)
     PAR4: 2.98 [W/m2K] k0 (k*F'=k0+k1*dT)
     PAR5: 0.01 [W/m2K2] k1     
     PAR6: 10 [] n:number of nodes in 1xn-model                          
     PAR7: 3.8 [kJ/kgK] specific capacity cp of fluid 
     PAR8: 45 [] slope
     PAR9: 0 [] orientation (0:south,90:west,-90:east)     
     PAR10: 7.78 [m] length of absorber tube
     PAR11: 6.8 [mm] inside diameter of tube
     PAR12: 30 [kJ/Km2] capacity per squaremeter
     PAR13: 0 [] F' (not yet used)                           
     PAR14: -0.12 [] b0 iam factor: iam=1-b0*(1/cos(teta)-1), if -1: take iam_table
     PAR15: 0 [] 0:no horizon 1:horizon from './horizon/horizon.dat'

     INP1: 0 [Ws] deltaQ-inlet
     INP2: 0 [kg/s] mp-inlet
     INP3: 10.0 [C] ambient temperature
     INP4: 0 [W/m^2] Igh: global horiz. radiation
     INP5: 0 [W/m^2] Idh: diffuse radiation horiz. 
     INP6: 0 [W/m^2] IgT: global rad. on tilted surf. (e.g. E_40)
     INP7: 0 [] incidence angle
     INP8: 0 [] slope (if variabel)
     INP9: 0 [] orientation (if variabel)

     OUT1: [Ws] deltaQ-outlet
     OUT2: [kg/s] mp-outlet
     OUT3: [C] T_n last node n
     OUT4: [C] T_in            
     OUT5: [W] P_use 

     OUT6: [Ws] system inlet energy 
     OUT7: [Ws] system outlet energy 
     OUT8: [Ws] internal energy 

     OUT9:  [K] T_mean
     OUT10: [K] deltaTambient
     OUT11: [W] thermal losses to ambient                             
     OUT12: [C] T1: temp node 1                              

     OUT13: [C] T2                    
     OUT14: [C] T3                            
     OUT15: [K/W] deltaTamb/E
     OUT16: []

     OUT17: [] 
     OUT18: [] 
     OUT19: []  
     OUT20: [] 

     OUT21: 
     OUT22: [] 
     OUT23: []    
     OUT24: [] i_dead_nr[0]

     OUT25: [] eta=Q_use/(E_ges)

     OUT26: [W] A*(IbT+IrT+IdT) 
     OUT27: [W/m^2] IbT beam rad. on tilted surf.
     OUT28: [W/m^2] IrT+IdT ground refl. on tilted surf.
     OUT29: [] angl_mod(teta)

     OUT30: rad proc
     OUT31: rad proc
     OUT32: rad proc teta
     OUT33: rad proc phi

     OUT34: [] Rd Ratio diffus (horizon)
     OUT35: [] Rr ration reflektiv (horizon) 
    
     DERIV1: 10 [] node1 in_tube
     DERIV2: 10 [] node2 in_absorber
     DERIV3: 10 [] node3 2.tube
     DERIV4: 10 [] node4 2._absorber
     DERIV5: 10 [] node5 3.tube
     DERIV6: 10 [] node6 3.absorber
     DERIV7: 10 [] node7 4.tube
     DERIV8: 10 [] node8 4.absorber

  */
  

#define N_MAX 100 

int a=0; /*---------------------------debug flag --------------*/
int pedantic=1;
double energy [N_MAX+1] ;
double Qi[N_MAX+1][5]; 

double F_kA[N_MAX+1]; /* F_kA heatremoval from absorber to fluid */
double deltaT[N_MAX+1];
double A=par[unit][1];
double b0=par[unit][14]; /* default inc angl mod */
double mp,mc,mc_i,mc_fluid,mc_fluid_i;
double albedo=0.2; /* ambient reflection */
double cp=par[unit][7]*1e+3;

double eta0     =par[unit][2];
double eta0_diff=par[unit][3];
double F_k0=par[unit][4];
double F_k1=par[unit][5];
int n = (int) par[unit][6]; /* 1xn nodes */

double inc_angl,shadow=0;

/* radiation processor */
static double *rad_proc; /* array for outputs */
double Ib_hor,Id_hor,E_40; /*radiation on horizontal surface */
double I_beam,I_diff;
double mp_cp_h;
double Rd=0,Rr=0; /* ratio diffus for horizon */
double slope=par[unit][8],orientation=par[unit][9];
double dQ_out;
double dQ_in=out_n[  in_u[unit][1] ]
                  [ in_nr[unit][1] ];
double Tamb =out_n[ in_u [unit][3] ]
                  [ in_nr[unit][3] ];
int i=0,j=0,n_p;
double Tmean=0;
double T_tmp,x_p,T_plug,t_dead,energy_with_plug;
static int local_init_flag=1;
 
if (local_init_flag) { rad_proc=malloc_1double(15); local_init_flag=0; }

mc=A*par[unit][12]*1e+3; /*  capacity total fluid and plate and ... */ 
mc_i=mc / (double)(n); /* n>=1 !! */
mc_fluid = par[unit][10]*par[unit][11]*par[unit][11]*3.14159*cp/1000.0;
mc_fluid_i = mc_fluid/( double)(n);

if (  ( in_u [unit][8] != unit ) && ( in_u [unit][8] != 0 ) ) /* inp connected */ 
      slope      =out_n[ in_u [unit][8] ] 
                       [ in_nr[unit][8] ];
if (  ( in_u [unit][9] != unit ) && ( in_u [unit][8] != 0 ) )
      orientation=out_n[ in_u [unit][9] ] /* inp connected */
                       [ in_nr[unit][9] ];

Ib_hor=out_n[ in_u [unit][4] ]
            [ in_nr[unit][4] ]
      -out_n[ in_u [unit][5] ]
            [ in_nr[unit][5] ] ;

Id_hor=out_n[ in_u [unit][5] ]
            [ in_nr[unit][5] ];  /* diffuse radiation */

E_40=  out_n[ in_u [unit][6] ]
            [ in_nr[unit][6] ];  /* E_40 */
inc_angl=1; /*later set in inc_angle_mod */
  

mp=out_n[  in_u[unit][2] ]
        [ in_nr[unit][2] ];
 
mp_cp_h=mp*cp*h;

for (i=1;i<=n;i+=1) 
   { 
   deltaT[i]=deriv_o[unit][i]-Tamb;                      
   F_kA[i]=A*(F_k0+F_k1*deltaT[i])/( (double) n );
   };


/*----------------- Initialisation of init_energy */
if (init_flag)
  {
  int i;
  /* init i_dead_nr*/
  i_dead_nr[0]+=4;
  out_o[unit][24]=out_n[unit][24]=i_dead_nr[0];

  if (deriv_o[unit][1]<=Tamb) deriv_o[unit][1]=deriv_n[unit][1]=Tamb;

  for (i=1;i<=n;i++) 
    {
    if (i>1) deriv_o[unit][i]=deriv_n[unit][i]=deriv_o[unit][1];
    init_energy[unit][i]=deriv_o[unit][i]*mc_i;
    }
     
    if (a)
    printf("T8_init:%.4f  energy8:%.4f \n"
     ,deriv_o[unit][8],init_energy[unit][8]);


  };  /* end of init----------------------------------------------*/
i_dead_nr[0]=(int) out_o[unit][24]; /*i_dead_nr*/


/*------------------------rad processing-------------------------------------*/
/* call horizon from './horizon/horizon_table.dat' */
if ( (par[unit][15]==1) && (init_flag) ) /* horizon from './horizon/horizon.dat' */
 {
    /* get Rd and Rr with horizon once */
  shadow=horizon(init_flag,rad_proc[1] /* teta_z */, rad_proc[2] /* gamma_s */
                 ,&Rd,&Rr,albedo,slope,orientation);
  out_n[unit][34]=out_o[unit][34]=Rd; /* storing in out */
  out_n[unit][35]=out_o[unit][35]=Rr;
 }
      
Rd=out_o[unit][34];
Rr=out_o[unit][35];

if (a) printf("collector rad_proc[1]:%.1f rad_proc[2]:%.1f \n",rad_proc[1],rad_proc[2]);

if ( in_u [unit][6] == unit ) /* E40 input not connected */
  rad_processor(tsim,day_of_year,Ib_hor,Id_hor,albedo,0 /*E_40*/
                ,par[unit][8] /*slope*/ ,par[unit][9] /*orient */,rad_proc
                ,lenght,latitude,Rd,Rr);
else
  rad_processor(tsim,day_of_year,0     ,/*Id_hor*/ E_40 ,albedo,0 /*E_40: not    jet used */ ,par[unit][8],par[unit][9],rad_proc,lenght,latitude,Rd,Rr);

    /* rad_proc[0..13] = output of rad_processor() ,marke*/
out_n[unit][27]=out_o[unit][27]=rad_proc[4]; /*IbT, later shadow correction */
    out_n[unit][28]=out_o[unit][28]=rad_proc[6]+rad_proc[5]; /* IrT+IdT,!later set again */
       
if (par[unit][15]==1)  /* horizon from './horizon/horizon.dat' */
  {
   if (a) printf("now horizon call: teta_z:%.2f gamma_s=azimut:%.2f\n"
                ,rad_proc[1],rad_proc[2]);
   shadow=horizon(init_flag,rad_proc[1] /* teta_z */, rad_proc[2] /* gamma_s */
                 ,&Rd,&Rr,albedo,slope,orientation);
   if (a) printf("Rd:%.3f Rr:%.3f  \n",Rd,Rr); 
   if (a) { printf("tsim:%.2f shadow:%.1f teta on surface:%.1f \n"
                  ,tsim/3600,shadow,rad_proc[10]); }
   if (fabs(shadow)>1) { printf("error shadow:%.2f \n",shadow);exit (-1);}
  }
else shadow=0;


/* b0:Flat collector -0.2699 , float glass, use default */
if (b0==0) { printf("b0 in collextor not set..using default -0.12 \n"); b0=-0.12; }
inc_angl=inc_angl_mod(init_flag,b0,rad_proc[3],rad_proc[11],iam);
out_n[unit][29]=out_o[unit][29]=inc_angl; 

/* get radiation */
if ( in_u [unit][6] == unit ) /* E40 input not connected */ 
  {
  I_beam  =out_o[unit][27]; /*=IbT    */
  I_diff  =out_o[unit][28]; /*IrT+IdT  */
  inc_angl=out_o[unit][29]; /* angle-modifier */
  }
else
  {
  if (init_flag) printf("Coll. unit:%d input E40!! \n",unit);
  I_diff  =out_n[ in_u [unit][5] ] 
                [ in_nr[unit][5] ]; /* get IdT direct from inp5 out_o[unit][28]     ; / * IrT+IdT */
  if ( (I_diff<-10) || (I_diff>1200) ) printf("coll u:%d Idiff:%.2f\n",unit,I_diff);
  if (I_diff>E_40) I_diff=E_40;
  I_beam  =E_40-I_diff; 
  if ( (I_beam<-50) || (I_beam>1300) )printf("coll u:%d I_beam:%.2f\n",unit,I_beam);
  if (I_beam<0)  I_beam=0;
  inc_angl=out_o[unit][29]; /* angle-modifier */
  if ( (inc_angl<0) || (inc_angl>1.4) ) printf("coll u:%d inc_angl:%.2f\n",unit,inc_angl);
  }
out_n[unit][26]=out_o[unit][26]=A*(I_beam*(1-shadow)+I_diff);

if (out_n[unit][26]<0) out_n[unit][26]=I_beam=I_diff=0;

/* ------------------------------------------------------------------------------------- */

if (mp_cp_h > 0) 
 {  
   t_dead = mc_fluid/mp_cp_h;   /* Dead time in collector pipe */
   x_p = (double)(n) / t_dead;  /* Endpoint of incoming plug in tube, in node numbers. */
   n_p =  (int)(x_p);
   T_plug = dQ_in/mp_cp_h;
 }
else t_dead=x_p=n_p=0;

/* ---------- Loop: Run through nodes of collector tube -------------------- */

for (i=1;i<=n;i+=1)  
 {  /* energy balance according to pipe3.c, but regarding additional absorber capacity */
   if (mp==0) 
   {
     Qi[i][3]=0;
     energy_with_plug = deriv_o[unit][i]*mc_i;
   }
   else       /* With mass flow: */
   {
     if (i==1)           /* First node: Only energy from plug enters */
     {
      Qi[i][3] = dQ_in;  /* Incoming energy */
      if (x_p>=1) energy_with_plug=T_plug*mc_fluid_i + deriv_o[unit][i]*(mc_i-mc_fluid_i);
      else        energy_with_plug=T_plug*   mp_cp_h + deriv_o[unit][i]*(mc_i-mp_cp_h);
     }
     else if (i <= n_p)  /* Here energy from plug+neighboring nodes enters */
     { 
       Qi[i][3] = dQ_in *(x_p-i+1) / x_p;      /* From plug */
       for (j=i-1;j>0;j--) Qi[i][3] += mc_fluid_i * deriv_o[unit][j]; /*From neighbors*/ 
       energy_with_plug = T_plug * mc_fluid_i + (mc_i-mc_fluid_i)*deriv_o[unit][i];
     }
     else if (i == n_p+1)  /* Plug ends in this node. */
     {			   /*Here neighbors also contribute to energy_with_plug */
       Qi[i][3] =  dQ_in * (x_p-i+1) / x_p;   
       for (j=i-1;j>0;j--) Qi[i][3] += mc_fluid_i * deriv_o[unit][j];
       energy_with_plug = T_plug *(mp_cp_h-n_p*mc_fluid_i) + 
                          deriv_o[unit][1] * ((n_p+1)*mc_fluid_i - mp_cp_h) + 
                          deriv_o[unit][i] * (mc_i - mc_fluid_i);
     }
     else              /* No energy from plug entering, only from neighbors */
     { 
       Qi[i][3] = deriv_o[unit][i-n_p-1]*(mp_cp_h-n_p*mc_fluid_i);
       for (j=n_p;j>0;j--) Qi[i][3] +=   mc_fluid_i * deriv_o[unit][i-j];
       energy_with_plug =  deriv_o[unit][i-n_p-1]*(mp_cp_h - n_p * mc_fluid_i) + 
                           deriv_o[unit][i-n_p  ]*((n_p+1)*mc_fluid_i-mp_cp_h) +
                           deriv_o[unit][i] * (mc_i - mc_fluid_i);
     }
   }
   T_tmp = energy_with_plug / mc_i;  /* Temporary new temperature of node */
   if (a) printf("T_tmp1[%d]:%.2f mp_cp_h:%.2f mc_i:%.2f energy_with_plug:%.2f\n"
                   ,i,T_tmp,mp_cp_h,mc_i,energy_with_plug);
   Qi[i][1]=(T_tmp - Tamb)  * F_kA[i]*h;
   /* check time constant due to kA_i */
   T_tmp=(energy_with_plug-Qi[i][1])/mc_i; /* New temp. with amb losses */
   if (   ( (Qi[i][1]>0) && (T_tmp<Tamb) ) /* instable */
       || ( (Qi[i][1]<0) && (T_tmp>Tamb) )
      )
   {
    if (pedantic) printf("coll[%d]:Euler stabilization T_tmp:%.1f Tamb:%.1f Qi:%.1f \n",
        unit,T_tmp,Tamb,Qi[i][1]);
    Qi[i][1]=energy_with_plug-Tamb*mc_i;/* set Tnode to Tamb */
   } 

   Qi[i][2]=A*(eta0*inc_angl*I_beam*(1-shadow)
              +eta0_diff*I_diff)*h/( (double) n ); /* Q_sun, Qgain*/
   if (a)
   { T_tmp = (energy_with_plug - Qi[i][1] + Qi[i][2]) /mc_i; /* 2nd. tmp-temp of node */
     printf("T_tmp2nd[%d]:%.2f energy_with_plug:%.1f\n",i,T_tmp,energy_with_plug);
   }
   qp_sum[unit][i][1]+=Qi[i][1];
   qp_sum[unit][i][2]+=Qi[i][2];
   qp_sum[unit][i][3]+=Qi[i][3];
   /*first evaluation of absorber-temp, to accelerate energy_flow node 1..2 ,etc. */
   if (a) printf("energy:%.1f Qi[%d][1]:%.2f Qi[%d][2]:%.2f Qi[%d][3]:%.2f Tamb:%.1f\n"
                 ,init_energy[unit][i],i,Qi[i][1],i,Qi[i][2],i,Qi[i][3],Tamb);
   if (a) printf("deriv_o[unit][%d]:%.2f deriv_o[unit][]:%.2f mp_cp_h:%.2f \n"
                ,i,deriv_o[unit][i],deriv_o[unit][i-1],mp_cp_h);
 } 

/* Calculate energy leaving the collector tube */
if (x_p < n)       /* if plug shorter than collector tube */
 { dQ_out = deriv_o[unit][n-n_p]*(mp_cp_h-n_p*mc_fluid_i);   /* dQout last element */
  for (j=0;j<n_p;j++) dQ_out += deriv_o[unit][n-j]*mc_fluid_i;
 }
else             /* if plug longer than tube */ 
 { dQ_out = T_plug * (mp_cp_h - n*mc_fluid_i);  
  for (j=1;j<=n;j++) dQ_out += deriv_o[unit][j]*mc_fluid_i;
 }
Qi[n+1][3]= dQ_out;
qp_sum[unit][n+1][3]+=Qi[n+1][3];

for (i=1;i<=n;i+=1) 
  {
  energy[i]=init_energy[unit][i]
           -qp_sum[unit][i  ][1]       /* Q_loss node to ambient */
           +qp_sum[unit][i  ][2]       /* Q_gain due to solar radiation */
           +qp_sum[unit][i  ][3]       /* Q_in fluid */
           -qp_sum[unit][i+1][3];      /* Q_out fluid */
  deriv_n[unit][i]=energy[i]/mc_i;
  if (deriv_n[unit][i]<(Tamb-10)) printf("coll[%d] T[%d]:%.1f<Tamb:%.1f\n"
                                       ,unit,i,deriv_n[unit][i],Tamb);
  Tmean+=deriv_n[unit][i]/( (double) n);
  /*  printf("Node:%d, Old Temp:%lf, New Temp:%lf\n",i,deriv_o[unit][i],deriv_n[unit][i]); */

  if (a) printf("deriv_n[unit][%d]:%.2f\n",i,deriv_n[unit][i]);
  };
 
                   
  if (a) printf("dQ_in:%.4f mp_cp_h:%.4f dQ_out:%.4f\n",dQ_in,mp_cp_h,dQ_out);
 

  /*-------------- set outputs -----------------------------*/

  out_n[unit][1]=dQ_out;  /* dQ_out */
  out_n[unit][2]=mp;
  out_n[unit][3]=deriv_n[unit][n];  /* T_n last node n */
  if ( fabs (deriv_n[unit][n])>300) 
        { printf("collector gau unit:%d Tabs:%.1f init_energy:%.2f,P71:%.1f, "
                 "P81:%.1f,P82:%.1f \n"
                 ,unit,deriv_n[unit][8],init_energy[unit][8],Qi[7][1]/h,
                 Qi[8][1]/h,Qi[8][2]/h);  exit (-1); }
  if (mp_cp_h>0) out_n[unit][4]=dQ_in/mp_cp_h; /* T_in */                   
  else           out_n[unit][4]=deriv_n[unit][1];

  out_n[unit][5]=(dQ_out-dQ_in)/h; /* P_use [W] */

  /*System Energy Balance--------------------------------------------- */
  out_n[unit][6]=0; for (i=1;i<=n;i++) out_n[unit][6]+=Qi[i][2]; /*sys gains*/
  out_n[unit][7]=0; for (i=1;i<=n;i++) out_n[unit][7]+=Qi[i][1]; /* losses */
  out_n[unit][8]=0; for (i=1;i<=n;i++) out_n[unit][8]+=energy[i]; /* cap */

   /*  T_mean  */
   out_n[unit][9]=Tmean;                   
   /*  deltaTambient */
   out_n[unit][10]=Tmean-Tamb;
   out_n[unit][11] = out_n[unit][7]/h; /* thermal losses to ambient */
   out_n[unit][12] = deriv_n[unit][1]; /* T1 */
   out_n[unit][13] = deriv_n[unit][2]; /* T2 */
   out_n[unit][14] = deriv_n[unit][3]; /* T3 */
    /* [K/W] deltaTamb/E */
   out_n[unit][15] = (Tmean-Tamb)/out_n[unit][26]; /* T3 */


   /*out25: eta=Q_use/(E_ges), out26: A*IgT*/
   if (out_n[unit][26]>10) out_n[unit][25] = out_n[unit][5]/(out_n[unit][26]+1e-12);
   else                    out_n[unit][25] = 0;

   /* don't use out_n[unit][24], it's  i_dead_nr[0] */

   /*  out26: earlier: IgT with shadow */
   /*  out27: IbT */
   out_n[unit][27]=out_n[unit][27]*(1-shadow); /* IbT shadow correction */
   /*  out28: IrT+IdT ground refl on tilted surface */
   /*  out29: angl_mod(teta)  */

   /* if min10_flag set by rad_proc[4,5,6] */

   out_n[unit][30] = rad_proc[1]; 
   out_n[unit][31] = rad_proc[2];  
   out_n[unit][32] = rad_proc[10];   /* teta */    
   out_n[unit][33] = rad_proc[11];   /* phi */      

   /* out 34 & 35 Rr,Rd */
return (1);
}
/*-------------------------------------------------------------------*/





