/* RC.c */
/*       10        20        30        40        50        60        70       */
/*3456789012345678901234567890123456789012345678901234567890123456789012345671*/
/*----------------------------------------------------------------------------*/


/*#include "global.h" */
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

/*------------------RC() --------------TYPE 23 -------------------------------*/

int RC(unit,par,in_u,in_nr,out_o,out_n,deriv_o,deriv_n,h,init_energy,
              qp_sum,tsim,init_flag)
           int    unit;      /* current unit number */
           double **par;     /* parameter array par[unit][1..PARMAX]  */
           int    **in_u;    /* input unit array, input connections   */
           int    **in_nr;   /* input number array                    */
           double **out_o;   /* output array with privious state      */
           double **out_n;   /* output array with current state       */
           double **deriv_o; /* derivatives: used for diffential equ. */
           double **deriv_n; /*  new state of deriv                   */
           double h;         /* sim time step */
           double **init_energy; /* initial energy in RC-comp. [Ws]   */
           double ***qp_sum; /* energy to one node qp[unit][n][m]     */
           double tsim;      /* simulation time in [s] of this day    */
           int    init_flag; /* set at initial loop of sim            */
{
/* used from ./etc/xfig.awk
 
   TYPE      23
   PRIORITY  12         used by config routine
   HYDRAULIC 99         used by config routine

   PAR1: 0.5 [kg] mc in kg water
   PAR2: 1 [W/K] kA1 to inp1
   PAR3: 1 [W/K] kA2 to inp2

   INP1: 20.0 [C] hot  ambient temperature
   INP2: 20.0 [C] cold ambient temperature

   OUT1: [C] Tnode ,Tsensor ... 
   OUT2: []
   OUT3: []
   OUT6: [Ws] system inlet energy 
   OUT7: [Ws] system outlet energy 
   OUT8: [Ws] internal energy

   DERIV1: 18 [C] temp of load_place

*/

 int a=0; /*-------------ANALYSE a=1 ---------*/
 double Q11,Q12;
 double mc=par[unit][1]*4190;
 double kA1=par[unit][2];
 double kA2=par[unit][3];

 double energy=0;

 /* Initialisation of init_energy */
 if (init_flag)
   {
     if ((fabs(par[unit][1])<0.2)) {printf("mc in RC unit:%d!!\n",unit);exit(-1);}
     init_energy[unit][1]=deriv_o[unit][1]*mc;

   }
 /* node energy flow */

 Q11=-kA1*(deriv_o[unit][1]-out_o[  in_u[unit][1] ]      /* flow to node (gains) */
                                 [ in_nr[unit][1] ])*h;
 Q12= kA2*(deriv_o[unit][1]-out_o[  in_u[unit][2] ]      /* flow from node (loss) */
                                 [ in_nr[unit][2] ])*h;

 /* integrated heat flow  qp_sum[unit][deriv][path] */
 qp_sum[unit][1][1]+=Q11;  /* gains */
 qp_sum[unit][1][2]+=Q12;  /* losses */

 energy= init_energy[unit][1]
             +qp_sum[unit][1][1]  /* Q_kA1 */
             -qp_sum[unit][1][2]; /* Q_kA2 */

 deriv_n[unit][1]=energy/mc;
 if ( (fabs(deriv_n[unit][1]-deriv_o[unit][1])>2.5) )
   printf("RC unit:%d warning temp-increase/decrease:%.2f \n"
          ,unit,deriv_n[unit][1]-deriv_o[unit][1]);
     
 /* if ( (fabs(out_o[  in_u[unit][1] ][ in_nr[unit][1] ]-deriv_o[unit][1])>5) )
     printf("RC unit:%d  warning: large temp-diff:%.2f \n",unit,
       fabs(out_o[  in_u[unit][1] ][ in_nr[unit][1] ]-deriv_o[unit][1]));
 */
 if (a) printf("RC unit:%d node temp:%.1f \n",unit,deriv_n[unit][1]);

/*--------------set outputs----------------------*/
/*dQ_loss_water */
out_n[unit][1]=deriv_n[unit][1]; /* node temp */

/*System Energy and Power */
out_n[unit][6]=Q11;       /* sys-in [Ws]*/
out_n[unit][7]=Q12;       /* sys out -Q12 if open end */
out_n[unit][8]=energy;    /* sys cap */

return 1;

}


