/* humidifier.c */
/*       10        20        30        40        50        60        70       */
/*3456789012345678901234567890123456789012345678901234567890123456789012345671*/
/*----------------------------------------------------------------------------*/

/*
010427 chriwi: some syntax
010627 include thermodynamics.h
020402 Tout xout defined, add includes
*/
#include "global.h"
#include "thermodynamics.h"
#include <stdio.h> 


int humidifier(unit,par,in_u,in_nr,out_o,out_n,deriv_o,deriv_n,h,init_energy
               ,qp_sum,tsim,init_flag,previous_unit,select_unit)
           int    unit;      /* current unit number */
           double **par;     /* parameter array par[unit][1..PARMAX]  */
           int    **in_u;    /* input unit array, input connections   */
           int    **in_nr;   /* input number array                    */
           double **out_o;   /* output array with privious state      */
           double **out_n;   /* output array with current state       */
           double **deriv_o; /* derivatives: used for diffential equ. */
           double **deriv_n; /*  new state of deriv                   */
           double h;         /* sim time step                         */
           double **init_energy; /* initial energy in RC-comp. [Ws]   */
           double ***qp_sum; /* energy to one node qp[unit][n][m]     */
           double tsim;      /* simulation time in [s] of this day    */
           int    init_flag; /* set at initial loop of sim            */
           int    previous_unit; /* coming from this unit             */
           int    *select_unit; /* number of calls in this timestep   */

{
/*  
   TYPE 56
   PRIORITY  6          used by config routine: pump priority
   HYDRAULIC 32         one pole air  

   PAR1: - [-]  sorry no parameter 

   INP1: 0 [Ws] 'Qin' inlet enthalpy
   INP2: 0 [kg/s] 'mp_dry_in' 
   INP3: 0 [kg/s] 'mp_steam_in'  mass stream steam .. auto-connection
   INP4: 0 [kg/s] 'mp_water_in'  mass stream water .. auto-connection
   INP5: 0 [kg/s] 'mp_co2_in'
   INP6: 0 [1] 'eta'  humidisfaction dx=eta*(xs_ideal-x);

   OUT1: [Ws] 'Q_out' output heat capacity
   OUT2: [kg/s] 'mp_dry_out' mass stream dry air,
   OUT3: [kg/s] 'mp_steam_out' 
   OUT4: [kg/s] 'mp_water_out'
   OUT5: [kg/s]  'mp_co2_out'

   OUT9: [C] 'Tout1'
   OUT10: [g/kg] 'x_out1' 
*/

double x,mp_steam_out;

double Q_in        =out_n IN(1);
double mp_dry_in   =out_n IN(2); /*[kg/s] mass stream dry air*/
double mp_steam_in =out_n IN(3);
double mp_water_in =out_n IN(4);
double mp_co2_in   =out_n IN(5);
double eta         =out_n IN(6);
double Tin,Tout,x_out;
double xs_ideal,dx;

if (mp_dry_in >0) {
 x=mp_steam_in/mp_dry_in;
 Tin=Get_Temp_from_enthalpy(Q_in,mp_dry_in*h,mp_steam_in*h,mp_water_in*h);
 xs_ideal=Get_xs_from_enthalpy(Q_in,mp_dry_in*h,mp_water_in*h,Tin);
 if (0) printf("xs_ideal=%8.7f \n",xs_ideal);
 dx=eta*(xs_ideal-x);
 mp_steam_out=mp_steam_in+mp_dry_in*dx;
} /* mp_dry_in >0 */
else {
 printf("humidifier[%2d]:mp_dry_in=0.0 \n",unit);
 mp_steam_out=mp_steam_in;
}
Tout=Get_Temp_from_enthalpy(Q_in,mp_dry_in*h,mp_steam_out*h,mp_water_in*h);
x_out=mp_steam_out/mp_dry_in;
if (0) printf("tsim %6.2f: humidifier[%2d] dx %5.3f xalt %7.5f  "
        "xneu %7.5f Tin %5.2f Tout %5.2f Q_in %8.1f mp_staem-in %6.5f "
        "xs_ideal %7.5f\n"
        ,tsim,unit,dx,x*1e3,x_out*1e3,Tin,Tout,Q_in,mp_steam_in,xs_ideal*1e3);

out_n[unit][1]=Q_in;
out_n[unit][2]=mp_dry_in;
out_n[unit][3]=mp_steam_out;
out_n[unit][4]=mp_water_in;
out_n[unit][5]=mp_co2_in;
out_n[unit][6]=0.0;
out_n[unit][7]=0.0;
out_n[unit][8]=0.0;
out_n[unit][9]=Tout;
out_n[unit][10]=x_out*1e3; /* [g/kg] */

return (1);
} /* humidifier() */

