/* pipe1.c */
/*       10        20        30        40        50        60        70       */
/*3456789012345678901234567890123456789012345678901234567890123456789012345671*/
/*----------------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "global.h"

/* pipe1 is used without deadtime-componentes with node model taken from
   collector1xn model . parameters inputs and outputs are similar to
   origin model pipe2.c, so you can use this instead by linking this one

000507 n converted to (double) n
       stability check due to kA_i 'stabilization'
000623 T_tmp=(energy_with_plug-Qi[i][1]+Qi[i][2])/(mp_cp_h+mc_i);
010517 80 columns limitations 
011128 labels to comment
020131 update vol,rho units, pipe_hyd() added 
       deadtime routine included of christiane kettner
*/

#define N_MAX 100  /* number of nodes */
int pipe_hyd(double *ceta,double l,double di,double m_dot,double rho,
             double Vol);
int DeadTimeFlow(double *energy_with_plug,double *dQ_out,double Qi[N_MAX+1][5],
                 int unit,int i,
                 int n,double **deriv_o, double mp_cp_h,double mc_i,double t_dead,
                 double dQ_in);


/*-------------------1xn PIPE1 -------------------------------PIPE -----------*/
int pipe2(unit,par,in_u,in_nr,out_o,out_n,deriv_o,deriv_n,h,init_energy,qp_sum,
          tsim,init_flag,dummy)
           int    unit;      /* current unit number */
           double **par;     /* parameter array par[unit][1..PARMAX]  */
           int    **in_u;    /* input unit array, input connections   */
           int    **in_nr;   /* input number array                    */
           double **out_o;   /* output array with privious state      */
           double **out_n;   /* output array with current state       */
           double **deriv_o; /* derivatives: used for diffential equ. */
           double **deriv_n; /*  new state of deriv                   */
           double h;         /* sim time step                         */
           double **init_energy; /* initial energy in RC-comp. [Ws]   */
           double ***qp_sum; /* energy to one node qp[unit][n][m]     */
           double tsim;      /* simulation time in [s] of this day    */
           int    init_flag; /* set at initial loop of sim            */
           double **dummy; /* internal out_n for pipe2 calls */
{
  /* following comment is used from script etc/xfig.awk   

    TYPE 31
    PRIORITY  6          used by config routine
    HYDRAULIC 1          1-pole fluid              

  
    PAR1: 1    [m]  'l' length l of pipe
    PAR2: 10   [mm] 'd' inside diameter di
    PAR3: 4.19 [kJ/kgK] 'cp' cp fluid (water) 
    PAR4: 0.04 [W/Km] 'kA/l'
    PAR5: 1.1  [1] 'f_cap' factor for fluid-capacity 
    PAR6: 10   [1] 'n' n_max nodes 
    PAR7: 1    [1] 'ceta' ceta,! this par will be over written 
    PAR8: 0 [1] 'hyd_dummy' ceta2 hyd
    PAR9: 1 [1] 'deadtime' 1:use deadtime model


    INP1: 0    [Ws] 'dQin' deltaQ-inlet
    INP2: 0    [kg/s] 'mdot_in' mp-inlet
    INP3: 20.0 [C]  'Tamb' ambient temperature
    INP4: 0    [Ws] 'Pgain' external gain
    
    OUT1: [Ws] 'dQout' dQ-out
    OUT2: [kg/s] 'mdot_out' mp-out
    OUT3: [C] 'Tout' outlet temperature 
    OUT4: [C] 'dTam'
    OUT5: [1] ' '
    OUT6: [Ws] 'sys-gain' system gains 
    OUT7: [Ws] 'sys-loss' system losses
    OUT8: [Ws] 'sys-ener' system capacity to to 0 degrees   
    OUT9: [C] 'T_mean'
    OUT10: [W] 'P_loss' ambient 
    OUT11: [C] 'T_in' T_inlet 
    OUT12: [Ws] 'dQamb' dQ_ambient Q11
    OUT13: [C]  'Tamb'

    OUT14: [kg/h]  'mdot_kg/h' used in hyd solve 
    OUT15: [Pa]  'dP' is written by hydraulic unit
    OUT16: [W/K] 'k_loss_eff'
    OUT17: [1] 'nop' empty
    OUT18: [kg/h] 'mdot_kg_h' mdot

    DERIV1: 10 [C] 'T1' temp of node 1
  */
  

int a=0; /*-----chriwi's debug flag-------*/
int i=0;
int pedantic=0;

double l=      PAR(1); /* lenght of pipe [m] */
double di=     PAR(2); /* diameter [mm] */
double cp=     PAR(3)*1e+3; /* [Ws/kgK] */
double f_cap=  PAR(5);/* [1] */
double kA_l=   PAR(4); /* kA/l */
int    n=(int) PAR(6); /* 1xn nodes */
double ceta=   PAR(7); /* this is written by pipe_hyd() */
/* double hyd_lin=PAR(8);  this is written by pipe_hyd() */
int deadtime_flag= (int) PAR(9); /* this is written by pipe_hyd() */


double dQ_in  =out_n IN(1); 
double mp     =out_n IN(2); 
double Tamb   =out_n IN(3);
double dQgains=out_n IN(4);

int hydraulic_flag=(int) (ceta>0); /* if!=0 */

double energy [N_MAX+1] ;
double Qi[N_MAX+1][5]; 
double deltaT[N_MAX+1];
double mc,mc_i;
double mp_cp_h;
double dQ_out;
double kA=l*kA_l;  /* kA-total */
double Vol=PI*di*di*l*1e-6/4.0; /* [m3] */
double rho=1000.0; /* kg/m3 */
double Tmean=0,T_tmp,energy_with_plug,kA_i;
double t_dead;

/* deadtime_flag=1; / * !!!!!!!!!!!!! */
 
/* if ( (dummy!=NULL) && (mp>0) ) a=1; / * debug */

if (f_cap>0.8) mc=f_cap*Vol*cp*rho; /* total capacity with */ 
else           mc=      Vol*cp*rho; /* ambient cap of par4 */
mp_cp_h=mp*cp*h; /* input-plug */

if (init_flag) {/*------------ Initialize -----------------------------------*/
  int i;
  if (n<1) { printf("unit:%d auto select par6 (n) \n",unit); par[unit][6]=n=5; }
  if (par[unit][5]<1.0) { 
    printf("pipe:%d auto select par5 (factor-fluid-capacity)\n"
           ,unit); f_cap=par[unit][5]=1.0; 
  } 
  if (deriv_o[unit][1]<=0) deriv_o[unit][1]=deriv_n[unit][1]=10;
  mc_i=mc/( (double) n ); 
  for (i=1;i<=n;i++) {

    if (i>1) deriv_o[unit][i]=deriv_n[unit][i]=deriv_o[unit][1]; /* init-temp */
    init_energy[unit][i]=deriv_o[unit][i]*mc_i;
    if (a) printf("init_energy[%d]:%.1f \n",i,init_energy[unit][i]);
  }
};  /* init_flag */

if (hydraulic_flag) pipe_hyd(&ceta,l,di,mp,rho,Vol); /* extern solver */
PAR(7)=ceta;  /* writing parameter !! */

mc_i=mc/( (double) n );       /* node-capacity if n>=1 !! */
kA_i=kA/( (double) n );

if (mp_cp_h>0) t_dead =mc/mp_cp_h;
else           t_dead =0;

/*------------------------------------------MAIN-----------------------------*/
for (i=1;i<=n;i+=1) { /* energy balance  n>=2 !! */

  if (deadtime_flag) { 
    DeadTimeFlow(&energy_with_plug,&dQ_out,Qi,unit,i,n,deriv_o,mp_cp_h,mc_i,t_dead,
                 dQ_in);
    T_tmp=energy_with_plug/mc_i;
  }
  else {
    if (i==1)       Qi[i][3]=dQ_in;
    else            Qi[i][3]=mp_cp_h*T_tmp; /* plug from previous node */ 
    energy_with_plug=Qi[i][3]+mc_i*deriv_o[unit][i]; /* get new mixed node */
    T_tmp=energy_with_plug/(mp_cp_h+mc_i);        /* get temperary temp of node */
  }

  if (a) printf("T_tmp1[%d]:%.2f mp_cp_h:%.2f mc_i:%.2f energy_with_plug:%.2f\n"
                ,i,T_tmp,mp_cp_h,mc_i,energy_with_plug);
  deltaT[i]=T_tmp-Tamb;
  Qi[i][1]=deltaT[i]*kA_i*h;          /* get node losses */
   /* check time constant due to kA_i */
  if (deadtime_flag) T_tmp=(energy_with_plug-Qi[i][1])/(mc_i); /*amb losses*/
  else               T_tmp=(energy_with_plug-Qi[i][1])/(mp_cp_h+mc_i);
  if (   ( (Qi[i][1]>0) && (T_tmp<Tamb) ) /* instable */
      || ( (Qi[i][1]<0) && (T_tmp>Tamb) ) 
     ) {
    if (pedantic) printf("pipe1[%d]:stabilization T_tmp:%.1f Tamb:%.1f "
                         "Qi:%.1f \n", unit,T_tmp,Tamb,Qi[i][1]);
    if (deadtime_flag) Qi[i][1]=energy_with_plug-Tamb*mc_i;
    else               Qi[i][1]=energy_with_plug-Tamb*(mp_cp_h+mc_i);/* set to Tamb*/
  } 
  Qi[i][2]=dQgains/( (double) n );     /* external gains */
  
  /*2nd.calc of node*/
  if (deadtime_flag) T_tmp=(energy_with_plug-Qi[i][1]+Qi[i][2])/mc_i;
  else               T_tmp=(energy_with_plug-Qi[i][1]+Qi[i][2])/(mp_cp_h+mc_i);

  if (a) printf("T_tmp2nd[%d]:%.2f energy_with_plug:%.1f\n" 
                ,i,T_tmp,energy_with_plug);
  qp_sum[unit][i][1]+=Qi[i][1];
  qp_sum[unit][i][2]+=Qi[i][2];
  qp_sum[unit][i][3]+=Qi[i][3];
} /* i++ */

if (deadtime_flag) Qi[n+1][3]=dQ_out; 
else               Qi[n+1][3]=dQ_out=mp_cp_h*T_tmp; /* dQout last element */

if (a) printf("pipe1: dQ_out:%.3f mp_cp_h:%.3f\n",dQ_out,mp_cp_h);
qp_sum[unit][n+1][3]+=Qi[n+1][3];
   
for (i=1;i<=n;i+=1) { /* get state */
  energy[i]=init_energy[unit][i]
                -qp_sum[unit][i  ][1]       /* Q_loss node 2 ambient */
                +qp_sum[unit][i  ][2]       /* Q_gain */
                +qp_sum[unit][i  ][3]       /* Q_in fluid */
                -qp_sum[unit][i+1][3];      /* Q_out fluid */
  deriv_n[unit][i]=energy[i]/mc_i;
  Tmean+=deriv_n[unit][i]/( (double) n );
  if ( (pedantic) && ( fabs (deriv_n[unit][i]-deriv_o[unit][i])>2) ) {
    printf("pipe unit:%d deriv_o[%d]:%.1f deriv_n[%d]:%.1f "
           "mp_cp_h/mc_i:%.1f tau_kA:%.1f sec\n",
            unit,i,deriv_o[unit][i],i,deriv_n[unit][i],mp_cp_h/mc_i,mc_i/kA_i);
  }
  if (a) printf("deriv_n[unit][%d]:%.2f\n",i,deriv_n[unit][i]);
  T_tmp=energy_with_plug/(mp_cp_h+mc_i); /* Input for next node */
} ; /* i++ */
                   
if (a) printf("dQ_in:%.4f mp_cp_h:%.4f dQ_out:%.4f energy10:%.1f\n"
              ,dQ_in,mp_cp_h,dQ_out,energy[10]);
if (a) printf("pipe%d:mp:%2f dQ_out:%2f mp_cp_h:%.2f T10:%.1f\n",unit,mp*3600
               ,dQ_out,mp_cp_h,deriv_o[unit][n]);
/* --------------WARNINGS-----------------------------------------------------*/
if ( (pedantic) && ( fabs(t_dead)>100.0*(double) (n)/h) ) {
 printf("pipe1:%d deadtime=%.1f [s]>100*n/h=%.1f [s] !!\n"
         ,unit,t_dead,100.0*(double) (n)/h);

 }
if ( fabs (deriv_n[unit][n])>300) 
        { printf("pipe gau unit:%d init_energy[%d]:%.2f,T[%d]:%.1f \n"
                 ,unit,n,init_energy[unit][n],n,deriv_n[unit][n]); exit (-1); }
/*-----------------------------------END of-MAIN-----------------------------*/

if (dummy!=NULL) /* check caller of pipe */
 {
 if (a) printf("internal call of pipe2: unit:%d\n",unit);
 out_n=dummy;
 }
if (isnan(dQ_out)) { 
 printf("pipe[%d]: dQ_out=NAN \n",unit); exit (-1); 
}


/*-------------- set outputs -----------------------------*/
out_n[unit][1]=dQ_out;  /* dQ_out */
out_n[unit][2]=mp;
/*out_n[unit][3]=deriv_n[unit][n];  outlet temperature T_n last node n */
if (mp!=0)
out_n[unit][3]=dQ_out/mp_cp_h;  /* mean outlet temperature */
else
out_n[unit][3]=deriv_n[unit][n];

if (mp_cp_h>0) out_n[unit][4]=dQ_in/mp_cp_h; /* T_in */                   
else           out_n[unit][4]=deriv_n[unit][1];

out_n[unit][4]=Tmean-Tamb; /*  deltaTambient */
out_n[unit][5]=(dQ_out-dQ_in)/h; /* P_loss [W] */

/*System Energy Balance--------------------------------------------- */
out_n[unit][6]=0; for (i=1;i<=n;i++) out_n[unit][6]+=Qi[i][2]; /* gains[Ws] */
out_n[unit][7]=0; for (i=1;i<=n;i++) out_n[unit][7]+=Qi[i][1]; /* sys losses */
out_n[unit][8]=0; for (i=1;i<=n;i++) out_n[unit][8]+=energy[i];

out_n[unit][9]=Tmean;                   
out_n[unit][10]=out_n[unit][7]/h;
if (mp_cp_h>0) out_n[unit][11] = dQ_in/mp_cp_h;         /* Tin */
else           out_n[unit][11] = deriv_n[unit][1];

out_n[unit][12] = out_n[unit][7] ; /* dQ_ambient Q11 */
out_n[unit][13] = Tamb;
out_n[unit][14] = mp*3600;

out_n[unit][16] = kA;
out_n[unit][18] = mp*3.6e3;


return (1);
}
/*----------------------------------------------------------------------------*/
int pipe_hyd(double *ceta,double l,double di,double m_dot,double rho,
             double Vol)
{
double A;
double Re;
double v;
double hyd_lin;
double ceta_intern=0;
 
A=Vol/l;
v=fabs(m_dot)/(rho*A);
Re=rho*v*di; 
if (Re<0) { /* ??? kommt nicht vor !!! */
   ceta_intern=(0.02+5e-4/(di*1.e-3))*l/(di*1.e-3)/(A*A);
   hyd_lin=32.0*l*1.0e-3/(A*di*di)*1.e6/rho;
}
if (Re>=2300)  {
 double logdummy;
 logdummy=(log10(Re/7.0));
 ceta_intern=l* (0.309/(logdummy*logdummy) /(di*1.0e-3*A*A));
 hyd_lin=0;

}
else { /* Re<2300 */
 double logdummy;
 Re=2300.0; /* !!! */
 logdummy=(log10(Re/7.0));
 ceta_intern=l*(0.309/(logdummy*logdummy) /(di*1.0e-3*A*A));
}
*ceta=ceta_intern;


return (1);
} /* int pipe_hyd() */

int DeadTimeFlow(double *energy_with_plug,double *dQ_out,double Qi[N_MAX+1][5],
                 int unit,int i,
                 int n,double **deriv_o, double mp_cp_h,double mc_i,double t_dead,
                 double dQ_in) 
{
/*   
model: christiane kettner 
   pipe3 is a node model according to the following concept:
   The incoming plug displaces the fluid in the pipe up to its own lenght
   in the pipe. From the pipe entrance til this lenght (x_p), all nodes have
   the temperature of the plug. All other nodes receive the fluid displaced
   by the plug. Fluid mixture takes place only within each node, not inbetween
   nodes.
   Therefore, by the number of nodes, the deadtime behaviour can be varied:
   Although the deadtime itself always equals the passage time through the
   pipe, the response curve to a sudden increase in plug temperature can be
   a step-function for an infinite number of nodes or a smooth curve for 
   fewer nodes.   Recommended are about 5-10 nodes.
   Note that although energies and temperatures are only stored for the
   nodes,  the calculation is accurate also for plugs of non-integer lenght
   (in units of nodes). 
*/


static int n_p;
static double T_plug,x_p;
int j;

if (i==1) { /* calc only once ,store to static variables */
 if (mp_cp_h>0) {
   x_p = (double)(n) / t_dead;  /* endpoint of the incoming plug in the pipe */
   n_p = (int)(x_p);
   T_plug=dQ_in/mp_cp_h;  /* Plug temperature */
 }
 else { 
   x_p = 0;
   n_p=0;
 }
} /* i==1 */

if (mp_cp_h==0) {  /* Without incoming flow */
  Qi[i][3]=0; *energy_with_plug=deriv_o[unit][i]*mc_i;
} 
else { /* mp_cp_h>0 */
 int case_condition=0;
 if (i==1) case_condition=1;
 else { 
   if (i<=n_p) case_condition=2;  /* plug+neighboring nodes enters here */
   else { if (i==(n_p+1))  {            /* Same as above, but neighbors also */
            case_condition=3;
          } 
          else case_condition=4; /* No contribution from plug, only from neighbors */
   } /* i>n_p */
 } /* i!=1*/

switch (case_condition) {
 case 1:
  Qi[i][3] = dQ_in;  /* Incoming energy */
  *energy_with_plug=((x_p>=1)
                    ?T_plug*mc_i
                    :T_plug*mp_cp_h+deriv_o[unit][i]*(mc_i-mp_cp_h));
  break;
 case 2:
   Qi[i][3]=dQ_in/x_p*(x_p-i+1);      /* From plug */
   for (j=i-1;j>0;j--) Qi[i][3]+=mc_i*deriv_o[unit][j]; /*neighbors*/
   *energy_with_plug=T_plug*mc_i;
  break;
 case 3:
  Qi[i][3]=dQ_in/x_p*(x_p-i+1);  /* contribute to energy_with_plug */
  for (j=i-1;j>0;j--) Qi[i][3]+=mc_i*deriv_o[unit][j];
  *energy_with_plug=T_plug *(mp_cp_h-n_p*mc_i)
                    +deriv_o[unit][1]*((n_p+1)*mc_i-mp_cp_h);
  break;
 case 4:
   Qi[i][3]=deriv_o[unit][i-n_p-1]*(mp_cp_h-n_p*mc_i);
   for (j=n_p;j>0;j--) Qi[i][3]+=mc_i*deriv_o[unit][i-j];
   *energy_with_plug=deriv_o[unit][i-n_p-1]*(mp_cp_h-n_p*mc_i)
                    +deriv_o[unit][i-n_p]*((n_p+1)*mc_i-mp_cp_h);
  break;
 default: printf("pipe[%d]: exit on case_condition:%d in DeadTimeFlow()\n"
                 ,unit,case_condition); exit (-1);
} /* case_condition */

} /*  mp_cp_h>0 */

/* Calculate energy leaving the pipe */
if (i==n) {
 if (x_p<n) {/* if plug shorter than pipe */
  *dQ_out = deriv_o[unit][n-n_p]*(mp_cp_h-n_p*mc_i);   /* dQout last element */
  for (j=0;j<n_p;j++) *dQ_out+=deriv_o[unit][n-j]*mc_i;
 }
 else { /* plug longer than pipe */
   *dQ_out=T_plug*(mp_cp_h-n*mc_i);
   for (j=1;j<=n;j++) *dQ_out+=deriv_o[unit][j]*mc_i;
 }

} /* i==n */

return (1);
} /* DeadTimeFlow() */

