/*  shading.c */
/*       10        20        30        40        50        60        70      */
/*345678901234567890123456789012345678901234567890123456789012345678901234567*/
/*---------------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "global.h"

#define X_MAX 20   /* used for shadow */
#define Z_MAX 20
#define MAX_SHADING 100

int calc_corners_shading (double hight,double width,double gap,
                          double extension,double hight_shadow,
                          double extension_shadow,double *As);
int calculate_phi(int shading_index,double HIGHT_width,double WIDTH_hight,
                  double projection,double gap,
                  double LEFT_bottom_extension,double RIGHT_top_extension,
                  double *RIGHT_sky_phi,double *LEFT_ground_phi);

int calc_shaded_area(double theta_z,double gamma_s,double hight,double width,
                     double projection_oh,double gap_oh,
                     double left_extension_oh,double right_extension_oh,
                     double projection_wl,double gap_wl,
                     double top_extension_wl,double bottom_extension_wl,
                     double projection_wr,double gap_wr,
                     double top_extension_wr,double bottom_extension_wr,
                     double *shadow);

int get_g_value (int blinds,int window_ID,double theta_z,double gamma_s,
                 double *g_beam,double *g_diff);


int shading(info,interface,radiation)
           struct sim_struct *info; /* see global.h */
           struct interface_struct *interface;
           struct radiation_struct *radiation;
{
/*  etc/xfig.awk

    TYPE 61
    PRIORITY  13         used by config routine: controller priority
    HYDRAULIC 99         used by config routine: non hydraulic

    PAR1: 0 [m] 'hight' hight of window (TWD)
    PAR2: 0 [m] 'width' width of window (TWD)

    PAR3: 0 [m] 'projection_oh' projection of overhang
    PAR4: 0 [m] 'gap_oh' gap of overhang
    PAR5: 0 [m] 'left_extension_oh' left extension of overhang
    PAR6: 0 [m] 'right_extension_oh' right extension of overhang

    PAR7: 0 [m] 'projection_wl' projection of left wingwall
    PAR8: 0 [m] 'gap_wl' gap of left wingwall
    PAR9: 0 [m] 'top_extension_wl' top extension of left wingwall
    PAR10: 0 [m] 'bottom_extension_wl' bottom extension of left wingwall

    PAR11: 0 [m] 'projection_wr' projection of right wingwall
    PAR12: 0 [m] 'gap_wr' gap of right wingwall
    PAR13: 0 [m] 'top_extension_wr' top extension of right wingwall
    PAR14: 0 [m] 'bottom_extension_wr' bottom extension of right wingwall

    INP1: 0 [1] 'blinds' get g-value from file

    OUT1: 1 [1] 'connect' connection to window or TWD

020517 stebus: Debugged return on call by main, change in wall.c, too.
       wall_index was a wrong name, now its called shading_index, though in 
       reality its the unit number. It could be changed to type_index in 
       future, but be carefull, in info->type_index you will get the index 
       of the calling wall. One wall can have 2 shading-elements (TWD and 
       window), one shading-element allso can be coupled to more than 1 wall. 
*/
int a=0;  /* debug all */
int i=1;
static int local_init=1;
static double *phi_sky,*phi_ground;
static double **shaded_area[MAX_SHADING];
int    unit=radiation->shading_unit; /* current unit, written by calling wall */
int    table_flag=1; /* write shadow into a table during the init */
double hight, width;
double projection_oh, gap_oh, left_extension_oh,  right_extension_oh;
double projection_wl, gap_wl,  top_extension_wl, bottom_extension_wl;
double projection_wr, gap_wr,  top_extension_wr, bottom_extension_wr;
int    blinds;
int    window_ID = 2001;
double shadow=0;
double rho_shading=0.4;
double IdT, IrT;
int init_flag=info->init_flag;
int shading_index=radiation->shading_unit;
double orientation=radiation->orientation;
double theta_z=radiation->theta_z;
double gamma_s=radiation->gamma_s;
double IbT=radiation->Ibt;
double Ibh=radiation->Igh-radiation->Idh;
double Idh=radiation->Idh; 
double Rd=radiation->Rd;
double Rr=radiation->Rr;
double inc_angl=radiation->inc_angl;
double iam_sky=radiation->iam_diff;
double iam_ground=radiation->iam_diff;
double rho_g=radiation->rho_g;
double g_beam;
double g_diff;

if (a)printf("\nshading called by unit:%d shading_unit:%d wall_unit:%d\n"
 ,info->unit,radiation->shading_unit,radiation->wall_unit);
if (a)printf("orientation=%.10f\n",orientation);
/*if (info->unit==radiation->shading_unit) return (1);*/
if (radiation->wall_unit==0) return (1); /* if called by main() return */
/*else read parameters*/
if (a)printf("calc shading!\n");
  hight               =  interface[unit].parameter[1];
  width               =  interface[unit].parameter[2];
  projection_oh       =  interface[unit].parameter[3];
  gap_oh              =  interface[unit].parameter[4];
  left_extension_oh   =  interface[unit].parameter[5];
  right_extension_oh  =  interface[unit].parameter[6];
  projection_wl       =  interface[unit].parameter[7];
  gap_wl              =  interface[unit].parameter[8];
  top_extension_wl    =  interface[unit].parameter[9];
  bottom_extension_wl =  interface[unit].parameter[10];
  projection_wr       =  interface[unit].parameter[11];
  gap_wr              =  interface[unit].parameter[12];
  top_extension_wr    =  interface[unit].parameter[13];
  bottom_extension_wr =  interface[unit].parameter[14];
  blinds              =  interface[unit].parameter[15];

if (local_init) {
    phi_sky=malloc_1double(MAX_SHADING); /* stebus shading */
    phi_ground=malloc_1double(MAX_SHADING);
    for (i=0;i<MAX_SHADING;i++)
      {phi_sky[i]=phi_ground[i]=0;}
    local_init=0;
}
/*--------INIT-------*/
if ((init_flag)&&((phi_sky[shading_index]==0)&&(phi_ground[shading_index]==0)))
  {
  if((width<0)||(hight<0)) /* prevent negativ measurements */
    {
    printf("area to be shaded in unit[%d] has negativ measurements!\n",unit);
    exit(-1);
    }
  if((gap_oh<0)||(projection_oh<0)||
     (left_extension_oh+right_extension_oh<-width))
    {
    printf("overhang in unit[%d] has negativ measurements!\n",unit);
    printf("gap_oh=%.2f projection_oh=%.2f left_extension_oh=%.2f \n"
           "right_extension_oh=%.2f width=%.2f\n",gap_oh,projection_oh,
           left_extension_oh,right_extension_oh,width);
    exit(-1);
    }
  if((gap_wl<0)||(projection_wl<0)||
     (bottom_extension_wl+top_extension_wl<-hight))
    {
    printf("left wing wall in unit[%d] has negativ measurements!\n",unit);
    exit(-1);
    }
  if((gap_wr<0)||(projection_wr<0)||
     (bottom_extension_wr+top_extension_wr<-hight))
    {
    printf("right wing wall in unit[%d] has negativ measurements!\n",unit);
    exit(-1);
    }  
  if (((left_extension_oh>gap_wl)||(top_extension_wl>gap_oh))
     &&(projection_oh!=projection_wl)
     &&(projection_oh!=0)&&(projection_wl!=0))
    {
    printf("cannot calculate overlapping shadows of overhang and left ");
    printf("wing wall in unit[%d]!\n",unit);
    exit(-1);
    }
  if(((right_extension_oh>gap_wr)||(top_extension_wr>gap_oh))
     &&(projection_oh!=projection_wr)
     &&(projection_oh!=0)&&(projection_wr!=0))
    {
    printf("cannot calculate overlapping shadows of overhang and right ");
    printf("wing wall in unit[%d]!\n",unit);
    exit(-1);
    }
  if(hight*width>0);
    {
    if((projection_oh*(left_extension_oh+right_extension_oh+width))>0)
      {
      printf("calculate_phi for overhang of unit[%d]\n",unit);
      calculate_phi(shading_index,hight,width,projection_oh,gap_oh,
  		    left_extension_oh,right_extension_oh,phi_sky,phi_sky);
      }
    if((projection_wl*(bottom_extension_wl+top_extension_wl+hight))>0)
      {
      printf("calculate_phi for left wing of unit[%d]\n",unit);
      calculate_phi(shading_index,width,hight,projection_wl,gap_wl,
		    bottom_extension_wl,top_extension_wl,phi_sky,phi_ground);
      }
    if((projection_wr*(bottom_extension_wr+top_extension_wr+hight))>0)
      {
      printf("calculate_phi for right wing of unit[%d]\n",unit);
      calculate_phi(shading_index,width,hight,projection_wr,gap_wr,
                    bottom_extension_wr,top_extension_wr,phi_sky,phi_ground);
      }
    }
  if(1)printf("phi_sky unit[%d]=%.5f phi_ground unit[%d]=%.5f\n",
	      unit,phi_sky[shading_index],unit,phi_ground[shading_index]); 
  }
 
/*------table_flag-----*/
if((init_flag)&&(table_flag))
  {
  char file_name[40];
  FILE *file_pointer;
  if (shading_index>MAX_SHADING) 
    { 
    printf ("shading_index of shading[%d]>MAX_SHADING\n",unit);
    exit(-1);
    }
  shaded_area[shading_index]=malloc_2double(181,111);
  sprintf (file_name,"shadow_unit_%d.dat",unit);
  if ((file_pointer=fopen(file_name,"w"))==NULL)
    {printf("datafile '%s' open error \n",file_name);exit(-1); };
  fprintf (file_pointer,"\n#shadow of unit[%d]\n",unit);
  if (1)printf("shadow of unit[%d]\n",unit);
/*
  fprintf (file_pointer,"#        -85    -75    -65    -55    -45    -35  "
           "  -25    -15     -5      5     15     25     35     45     55 "
           "    65     75     85\n");
*/
  fprintf (file_pointer,"#theta_z gamma_s shadow\n");
  if (a)printf("#theta_z gamma_s shadow\n");
  for (theta_z=0;theta_z<=110;theta_z+=1)
   {
   for (gamma_s=-90;gamma_s<=90;gamma_s+=1)
     {
     calc_shaded_area(theta_z,gamma_s,hight,width,projection_oh,gap_oh,
                      left_extension_oh,right_extension_oh,projection_wl,
                      gap_wl,top_extension_wl,bottom_extension_wl,
                      projection_wr,gap_wr,top_extension_wr,
                      bottom_extension_wr,&shadow);
/*   fprintf (file_pointer,"%.4f ",*shadow);  */
     fprintf (file_pointer,"%.0f %.0f %.4f\n",theta_z,gamma_s,shadow);
     shaded_area[shading_index][(int) (gamma_s+90)][(int) (theta_z)]=shadow;
     if (a)printf("%.0f %.0f %.4f\n",theta_z,gamma_s,
                  shaded_area[shading_index][(int) (gamma_s+90)][(int) (theta_z)]);
     }
   fprintf (file_pointer,"\n");
   if (a)printf("\n");
   }
  fclose(file_pointer);
  }
/*------END INIT-----*/

if((!init_flag)&&(!table_flag)&&(IbT>0))
  {
  if(a)printf("IbT=%.4f\n",IbT);
  calc_shaded_area(theta_z,gamma_s-orientation,hight,width,projection_oh,
	           gap_oh,left_extension_oh,right_extension_oh,projection_wl,
		   gap_wl,top_extension_wl,bottom_extension_wl,projection_wr,
		   gap_wr,top_extension_wr,bottom_extension_wr,&shadow);
  }
if((!init_flag)&&(table_flag)&&(IbT>0)) /* read shadow from table */
  {
  if (gamma_s<-90) { gamma_s=-90; printf("shading[%d]:gamma_s<-90!\n",unit); }
  if (gamma_s> 90) { gamma_s= 90; printf("shading[%d]:gamma_s> 90!\n",unit); }
  if (theta_z>110) { theta_z=110; printf("shading[%d]:theta_z>110!\n",unit); }
  shadow=shaded_area[shading_index][(int) (gamma_s+90.5)][(int) (theta_z+0.5)];
  if(a)printf ("gamma_s=%.4f theta_z=%.4f shadow unit[%d]=%.4f\n",
               gamma_s,theta_z,unit,shadow);
  }
if (a) printf("shading: IbT=%.2f shadow=%.2f Rd=%.2f Rr=%.2f inc_angl=%.2f\n",
                IbT,shadow,Rd,Rr,inc_angl); 
IdT= Idh*     (Rd-phi_sky[shading_index])
   +(Ibh+Idh)*rho_g*rho_shading*phi_sky[shading_index];
IrT=(Ibh+Idh)*(Rr-phi_ground[shading_index]*rho_g
   +          rho_g*rho_shading*phi_ground[shading_index]);
/* +IbT_wingwall*rho_shading*shadow_wingwall / * beam on shading element */

radiation->Ibt_trans=inc_angl*IbT*(1-shadow);
radiation->Idt_trans=IdT*iam_sky+IrT*iam_ground;
radiation->Igt_trans=radiation->Ibt_trans+radiation->Idt_trans;

if (a) printf("shading: Ibt_trans=%.2f Idt_trans=%.2f\n",
              radiation->Ibt_trans,radiation->Idt_trans);

if(a)printf("IbT=%.4f IdT=%.4f IrT=%.4f Ibt_trans=%.4f Idt_trans=%.4f\n",
            IbT,IdT,IrT,radiation->Ibt_trans,radiation->Idt_trans);

if (blinds!=0) get_g_value(blinds,window_ID,theta_z,gamma_s,&g_beam,&g_diff);

return (0);
} /* shading() */

/*---- calculate_phi --------------------------------------------------------*/
int calculate_phi(int shading_index,double HIGHT_width,double WIDTH_hight,
		  double projection,double gap,
		  double LEFT_bottom_extension,double RIGHT_top_extension,
		  double *RIGHT_sky_phi,double *LEFT_ground_phi)
/* if phi for the overhang is calculated "LEFT_bottom" means left, 
   "RIGHT_sky" means right etc.
   if phi for a wing is calculated "LEFT_bottom" means bottom,
   "RIGHT_sky" means sky etc. */
{   
int vdi=1; /* control */
double pi=3.141593;
int x, z;
double x_position, z_position;
double A_1, dA_1, A_2;
double a, b, c;
double phi_1_21, phi_1_22, phi_1_21sum=0, phi_1_22sum=0;
double phi_1_2, phi_2_1, phi_2_1sum=0;
for(x=1;x<=X_MAX;x++) /*run through window*/
  {
  x_position=HIGHT_width/X_MAX*(x-0.5); /*from BOTTOM_left to TOP_right*/
  for(z=1;z<=Z_MAX;z++)
    {
    z_position=WIDTH_hight/Z_MAX*(z-0.5); /*from LEFT_bottom to RIGHT_top*/
    if(0)printf("position=[%.3f|%.3f] ",x_position,z_position);
    a=gap+x_position;
    b=WIDTH_hight-z_position+RIGHT_top_extension;
    c=projection;
    if(0)printf("B=b/a=%.4f C=c/a=%.4f" ,b/a,c/a);
    if(vdi)
      {
      A_1=WIDTH_hight*HIGHT_width;
      dA_1=WIDTH_hight/Z_MAX*HIGHT_width/X_MAX;
      A_2=(LEFT_bottom_extension+WIDTH_hight+RIGHT_top_extension)
           *projection;
      }
    phi_1_21=1/(2*pi)*
      (atan(b/a)-1/sqrt(1+(c/a)*(c/a))*atan(b/a/sqrt(1+(c/a)*(c/a))));
    if(b>0)phi_1_21sum+=phi_1_21; /* view upwards, cf. TRNSYS */
    else   phi_1_22sum-=phi_1_21; /* top of wing lower than window */
    if(0)printf("phi_1_21=%.4f ",phi_1_21*b/fabs(b));
    b=LEFT_bottom_extension+z_position;
    phi_1_22=1/(2*pi)*
      (atan(b/a)-1/sqrt(1+(c/a)*(c/a))*atan(b/a/sqrt(1+(c/a)*(c/a))));
    if(b>0)phi_1_22sum+=phi_1_22; /* view downwards, cf. TRNSYS */
    else   phi_1_21sum-=phi_1_22; /* bottom of wing higher than window */
    if(0)printf("phi_1_22=%.4f\n",phi_1_22);
    if(vdi) /* VDI Waermeatlas Ka 4 */
      {
      phi_1_2=phi_1_21+phi_1_22; /* Summationsbeziehung (14) */
      phi_2_1=phi_1_2*dA_1/A_2; /* Reziprozitaetsbeziehung (12) */
      phi_2_1sum+=phi_2_1; /* Aufsummieren von phi_2_1 */
      }
    }
  }
phi_1_21=phi_1_21sum/(X_MAX*Z_MAX);
phi_1_22=phi_1_22sum/(X_MAX*Z_MAX);
RIGHT_sky_phi[shading_index]+=phi_1_21;
LEFT_ground_phi[shading_index]+=phi_1_22;
if(1)printf("phi_1_21=%.4f phi_1_22=%.4f phi_1_21+phi_1_22=%.4f\n",
            phi_1_21,phi_1_22,(phi_1_21+phi_1_22));
if(vdi)
  {
  phi_1_2=phi_2_1sum*A_2/A_1;
  printf("nach VDI Waermeatlas Ka 4: phi_1_2=%.4f\n",phi_1_2);
  }
if (vdi&&(LEFT_bottom_extension==0)&&(RIGHT_top_extension==0))/* control -> */
  {
  double phi_21_1, A_21, B, C; /* VDI Waermeatlas Kb 7 (18) */
  a=WIDTH_hight;
  b=projection;
  c=HIGHT_width+gap;
  B=b/a;
  C=c/a;
  phi_1_2=1/(pi*B)*(
                   B*atan(1/B)+C*atan(1/C)
                   -sqrt(B*B+C*C)*atan(1/sqrt(B*B+C*C))
                   +0.25*(
                          B*B*log((1+B*B+C*C)*B*B/((B*B+C*C)*(1+B*B)))
                         +C*C*log((1+B*B+C*C)*C*C/((B*B+C*C)*(1+C*C)))
                             -log((1+B*B+C*C)    /((1  +B*B)*(1+C*C)))
              	         )
		   );
  if(gap!=0)
    {
    c=gap;
    B=b/a;
    C=c/a;
    phi_1_22=1/(pi*B)*(
	              B*atan(1/B)+C*atan(1/C)
          	      -sqrt(B*B+C*C)*atan(1/sqrt(B*B+C*C))
           	      +0.25*(
		             B*B*log((1+B*B+C*C)*B*B/((B*B+C*C)*(1+B*B)))
                            +C*C*log((1+B*B+C*C)*C*C/((B*B+C*C)*(1+C*C)))
                                -log((1+B*B+C*C)    /((1  +B*B)*(1+C*C)))
			    )
	              );
    phi_1_21=phi_1_2-phi_1_22;
    }
  else phi_1_21=phi_1_2;
  A_1=WIDTH_hight*projection;
  A_21=WIDTH_hight*HIGHT_width;
  phi_21_1=phi_1_21*A_1/A_21;
  printf("nach VDI Waermeatlas Kb 7: phi_21_1=%.4f\n",phi_21_1);
  } /* <- control */
return(0);
}
/*------ calc_shaded_area ---------------------------------------------------*/
int calc_shaded_area(double theta_z,double gamma_s,double hight,double width,
	    	     double projection_oh,double gap_oh,
		     double left_extension_oh,double right_extension_oh,
		     double projection_wl,double gap_wl,
            	     double top_extension_wl,double bottom_extension_wl,
            	     double projection_wr,double gap_wr,
		     double top_extension_wr,double bottom_extension_wr,
		     double *shadow)
{
int a=0;  /* debug all */
double pi=3.141593;
double rad=pi/180;
double As;
double hight_shadow_oh;
double width_shadow_wl;
double width_shadow_wr;
double left_extension_shadow;
double right_extension_shadow;
double bottom_extension_shadow;
double top_extension_shadow;
double As_without_corners;

theta_z+=1e-10;  /* prevents undefined tan(90) or tan(-90) */
gamma_s+=1e-10;  /* prevents deviding by zero */

As=0;
if(a) printf("gamma_s-orientation=%.4f theta_z=%.4f\n",gamma_s,theta_z);
hight_shadow_oh=
  projection_oh/(tan(theta_z*rad)*cos((gamma_s/*-orientation*/)*rad));
if(a)printf("hight_shadow_oh=%.4f\n",hight_shadow_oh);
if(hight_shadow_oh>gap_oh)
  {
  left_extension_shadow=-projection_oh*tan((gamma_s/*-orientation*/)*rad);
  right_extension_shadow=projection_oh*tan((gamma_s/*-orientation*/)*rad);
  if(hight_shadow_oh-gap_oh<hight) 
    {
    As+=(hight_shadow_oh-gap_oh)*width;
    if(a)printf("overhang:   dAs=%.4f\n",((hight_shadow_oh-gap_oh)*width));
    }
  else 
    {
    As+=hight*width;					      /* overhang */
    if(a)printf("overhang:   dAs=%.4f\n",(hight*width));
    }
  if(a)printf("left edge\n");
  calc_corners_shading (hight,width,gap_oh,left_extension_oh,     /* left */
	  		hight_shadow_oh,left_extension_shadow,&As);
  if(a)printf("right edge\n");
  calc_corners_shading (hight,width,gap_oh,right_extension_oh,   /* right */
                        hight_shadow_oh,right_extension_shadow,&As); 
  if(a)printf("sum:         As=%.4f\n",As);
  }

width_shadow_wl=projection_wl*tan((gamma_s/*-orientation*/)*rad);
if(width_shadow_wl>gap_wl)
  {
  bottom_extension_shadow=
      projection_wl/(tan(theta_z*rad)*cos((gamma_s/*-orientation*/)*rad));
  top_extension_shadow=
     -projection_wl/(tan(theta_z*rad)*cos((gamma_s/*-orientation*/)*rad));
  if(width_shadow_wl-gap_wl<width) 
    {
    As+=(width_shadow_wl-gap_wl)*hight;
    if(a)printf("left wing:  dAs=%.4f\n",((width_shadow_wl-gap_wl)*hight));
    }
  else
    {
    As+=width*hight;		       			     /* left wing */
    if(a)printf("left wing:  dAs=%.4f\n",(width*hight));
    }
  if(a)printf("bottom edge\n");
  calc_corners_shading (width,hight,gap_wl,bottom_extension_wl, /* bottom */
                        width_shadow_wl,bottom_extension_shadow,&As);
  if(a)printf("top edge\n");
  calc_corners_shading (width,hight,gap_wl,top_extension_wl,       /* top */
                        width_shadow_wl,top_extension_shadow,&As);
  if(a)printf("sum:         As=%.4f\n",As);
  }

width_shadow_wr=-projection_wr*tan((gamma_s/*-orientation*/)*rad);
if(width_shadow_wr>gap_wr)
  {
  bottom_extension_shadow=
       projection_wr/(tan(theta_z*rad)*cos((gamma_s/*-orientation*/)*rad));
  top_extension_shadow=
      -projection_wr/(tan(theta_z*rad)*cos((gamma_s/*-orientation*/)*rad));
  if(width_shadow_wr-gap_wr<width)
    {
    As+=(width_shadow_wr-gap_wr)*hight;
    if(a)printf("right wing: dAs=%.4f\n",((width_shadow_wr-gap_wr)*hight));
    }
  else 
    {
    As+=width*hight;				            /* right wing */
    if(a)printf("right wing: dAs=%.4f\n",(width*hight));
    }
  if(a)printf("bottom edge\n");
  calc_corners_shading (width,hight,gap_wr,bottom_extension_wr, /* bottom */
                        width_shadow_wr,bottom_extension_shadow,&As);
  if(a)printf("top edge\n");
  calc_corners_shading (width,hight,gap_wr,top_extension_wr,       /* top */
                        width_shadow_wr,top_extension_shadow,&As);
  if(a)printf("sum:         As=%.4f\n",As);
  }
 
if(hight_shadow_oh>gap_oh) hight_shadow_oh-=gap_oh; else hight_shadow_oh=0;
if(hight_shadow_oh>hight) hight_shadow_oh=hight;
if(width_shadow_wl>gap_wl) width_shadow_wl-=gap_wl; else width_shadow_wl=0;
if(width_shadow_wl>width) width_shadow_wl=width;
if(width_shadow_wr>gap_wr) width_shadow_wr-=gap_wr; else width_shadow_wr=0;
if(width_shadow_wr>width) width_shadow_wr=width;

As_without_corners=hight*width
  -(hight-hight_shadow_oh)*(width-width_shadow_wl-width_shadow_wr);
if(a)printf("As_without_corners=%.4f*%.4f-%.4f*%.4f=%.4f\n",
         hight,width,(hight-hight_shadow_oh),
         (width-width_shadow_wl-width_shadow_wr),As_without_corners);

*shadow=As/(hight*width);
if(a) printf("shadow=%.4f\n",*shadow);
if((*shadow>1.001)||(*shadow<-0.001)) 
  {
  printf("shadow out of range, shadow=%.4f\n",*shadow);
  if((*shadow>1.01)||(*shadow<-0.01)) exit(-1);
  }
return (0);
}
/*--- calc_corners_shading  -------------------------------------------------*/
int calc_corners_shading (double hight,double width,double gap,
			  double extension,double hight_shadow,
			  double extension_shadow,double *As)
{
int a=0;  /* debug all */ 
double h=hight_shadow-gap;
double e=extension_shadow*h/hight_shadow;
double E=extension+extension_shadow-e;
double dAs;
if(h>hight)
  {
  e=e*hight/h;
  h=hight;
  }
if(a)printf("extension=%.4f extension_shadow=%.4f ",extension,extension_shadow);
if(a)printf("e=%.4f E=%.4f h=%.4f width=%.4f\n",e,E,h,width);
if((e<0)&&(E>0)&&(-e>E)&&(-e<E+width)) 
  {
  dAs=(E+0.5*e)*h+0.5*E*E*h/e;
  *As+=dAs;
  if(a)printf("case 1:     dAs=%.4f\n",dAs);
  }
if((e<0)&&(E<=0)&&(-E-e<=width))         
  {
  dAs=(E+0.5*e)*h;
  *As+=dAs;
  if(a)printf("case 2:     dAs=%.4f\n",dAs);
  }
if((e<0)&&(E>0)&&(-e>E+width))         
  {
  dAs=(E+0.5*e)*h+0.5*E*E*h/e-0.5*(e+E+width)*(e+E+width)*h/e;
  *As+=dAs;
  if(a)printf("case 3:     dAs=%.4f\n",dAs);
  }
if((e<0)&&(E<=0)&&(-E-e>=width)&&(-E<width))         
  {
  dAs=(E+0.5*e)*h-0.5*(e+E+width)*(e+E+width)*h/e;
  *As+=dAs;
  if(a)printf("case 4:     dAs=%.4f\n",dAs);
  }
if((e<0)&&(-E>=width))
  {
  dAs=-width*h;
  *As+=dAs;
  if(a)printf("case 5:     dAs=%.4f\n",dAs);
  }
if((e>0)&&(E<0)&&(e>-E)&&(-E<width))   
  {
  dAs=(E+0.5*e)*h-0.5*(e+E)*(e+E)*h/e;
  *As+=dAs;
  if(a)printf("case 6:     dAs=%.4f\n",dAs);
  }
if((e>0)&&(E<0)&&(e<=-E))               
  {
  dAs=(E+0.5*e)*h;
  *As+=dAs;
  if(a)printf("case 7:     dAs=%.4f\n",dAs);
  }
if((e>0)&&(E<0)&&(e>-E)&&(-E>width))   
  {
  dAs=-0.5*E*E*h/e+0.5*(E+width)*(E+width)*h/e;
  *As+=dAs;
  if(a)printf("case 8:     dAs=%.4f\n",dAs);
  }
if((e>0)&&(E<0)&&(e<-E)&&(-E>=width)&&(-E-e<width))   
  {
  dAs=-((E+width)*h+0.5*e/h+0.5*(E+width)*(E+width)*h/e);
  *As+=dAs;
  if(a)printf("case 9:     dAs=%.4f\n",dAs);
  }
if((e>0)&&(E<0)&&(-E-e>=width))
  {
  dAs=-width*h;
  *As+=dAs;
  if(a)printf("case 10:    dAs=%.4f\n",dAs);
  }
return(0);
}
 
/*-------------------------blinds---------------------------------------------*/

int get_g_value (int blinds,int window_ID,double theta_z,double gamma_s,
                 double *g_beam,double *g_diff)
{

return(0);
}

