/* wall.c */
/*       10        20        30        40        50        60        70       */
/*3456789012345678901234567890123456789012345678901234567890123456789012345671*/
/*----------------------------------------------------------------------------*/


#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "global.h"

/*-------------------------------------------WALL TYPE 28---------------------*/
int wall( unit,par,in_u,in_nr,out_n,out_o,deriv_o,deriv_n,h,
          init_energy,qp_sum,tsim,init_flag,type,
          day_of_year,lenght,latitude,iam
         )
           int    unit;      /* current unit number */
           double **par;     /* parameter array par[unit][1..PARMAX]  */
           int    **in_u;    /* input unit array, input connections   */
           int    **in_nr;   /* input number array                    */
           double **out_o;   /* output array with privious state      */
           double **out_n;   /* output array with current state       */
           double **deriv_o; /* derivatives: used for diffential equ. */
           double **deriv_n; /*  new state of deriv                   */
           double h;         /* sim time step                         */
           double **init_energy; /* initial energy in RC-comp. [Ws]   */
           double ***qp_sum; /* energy to one node qp[unit][n][m]     */
           double tsim;      /* simulation time in [s] of this day    */
           int    init_flag; /* set at initial loop of sim            */
           int *type;        /* type number array                     */
           int day_of_year;
           double *lenght;    /* geographic lenght                     */
           double *latitude;  /* geographic latitude                   */
           double **iam;     /* incidence angle modifier array        */

{
/*   xfig.awk
  
  TYPE 28
  PRIORITY  12         used by config routine
  HYDRAULIC 99         used by config routine


  
     PAR1: 0.880  [kJ/kgK] capacity of the wall
     PAR2: 0.93  [W/mK] Lambda of layer
     PAR3: 1900  [Kg/m3] density of layer
     PAR4: 15 [m] length of layer
     PAR5: 3 [m] height of layer
     PAR6: 0.4 [m] thickness of layer
     PAR7: 0.3  [] epsilon inside wall
     PAR8: 0.3  [] epsilon outside wall
     PAR9: 90   [] slope
     PAR10: -135 [] orientation
     PAR11: 10   [] number of layers
     PAR12: 0.8 [] Epsilon ambient

     INP1: 0 [Ws] Qrad in from room
     INP2: 10 [Ws] Qconv in from room
     INP3: 10 [C] T_outside
     INP4: 0 [] Ibh
     INP5: 0 [] Idh
     INP6: 0 [] E_40
     INP7: 1 [m/s] v-wind
     
     OUT1: [C] T_inside surface
     OUT2: []  
     OUT3: [] eps
     OUT4: [C] slope
     OUT5: [C] orientation
     OUT6: [Ws] system inlet energy 
     OUT7: [Ws] system outlet energy 
     OUT8: [Ws] internal energy 
     OUT9: [C] T1 
     OUT10: [C] T10
     OUT11: [] 
     OUT12: [W] IgT on wall
     
     DERIV1: 10 [C] temperature of node 1

*/

#define N 10   /* max number of nodes */

int a=0; /*-------------------analyse a=1--------------*/
int i=0;
int n=(int) par[unit][11];
 
double Qi [N+2] [3]; /* heat flow to nodes */
double energy [N+1] ;
double rad_proc[13]; /* array for outputs */

double kA;
double mc;
double slope=par[unit][9];
double orientation=par[unit][10];

double L=par[unit][4];              /* [m] lenght of wall */ 
double H=par[unit][5];              /* Height of wall */
double s=par[unit][6];              /* thickness of wall */
double A=L*H;                       /* Area of wall */   
double eps=par[unit][7];            /* Innenwand Epsilon */
double eps_w=par[unit][8];          /* Aussenwand Epsilon */
double eps_u=par[unit][12];         /* Epsilon Umgebung */

double g=9.81;  /* m/s2 */
double sigma=5.67e-8; /* */

double IgT,IdT,IT;

/* T1..T9 entspricht  DERIV[UNIT][1..9] */ 
/* NO-Wand */

double Qrad=out_n[in_u [unit][1] ]  /* get current Qrad from AirRadnode */
                 [in_nr[unit][1] ];

double Qconvi=out_n[in_u [unit][2] ]  /* get current Qconv from AirRadnode */
                   [in_nr[unit][2] ];

double T_inside =deriv_n[in_u[unit][1]][1];

double T_outside=out_o[in_u [unit][3] ]  
                      [in_nr[unit][3] ];
double Ibh      =out_o[in_u [unit][4] ]
                      [in_nr[unit][4] ];
double Idh      =out_o[in_u [unit][5] ]
                      [in_nr[unit][5] ];
double E_40     =out_o[in_u [unit][6] ]
                      [in_nr[unit][6] ];
double v        =out_o[in_u [unit][7] ]
                      [in_nr[unit][7] ];  /* m/s velocity of wind */
                    
/* Konvektiver bergang zu Aussenluft mit erzwungener */ 
/* Konvektion durch Wind nach Duffie (Feist:thermische Gebudesimulation) */
double alpha_conva=2.8+3*v;
double Qconva;
double Rd=0,Rr=0; /* external ratio of radiance for rad_proc */
double albedo =0.2; /* ground reflexion */

/* Langwelliger Strahlungsausgleich (Clarke,S.194) */

double Tsky;
double Qrad_outside;
double Qsol; /* solar energy */ 

/* Qconvi=h*alpha_convi*A*(T_inside -deriv_o[unit][1]);
*/
Qconva=h*alpha_conva*A*(T_outside-deriv_o[unit][n]);
/* Achtung nur bei sehr klarem Himmel gltig (Hochgebirge)!!!!! */
if (0) Tsky=pow(5.31e-13*pow(T_outside+273,6)/sigma,0.25);
else   Tsky=T_outside-20;

Qrad_outside=h*eps_w*A*sigma*(0.5*pow(Tsky,4)+0.5*eps_u*pow(T_outside+273+4,4)
             -pow(deriv_o[unit][n]+273,4));
mc      =A*s*par[unit][1]*1e3*par[unit][3]/n; /* cp*rho */
kA      =A*par[unit][2]*n/s;                  /*lambda*/

/*----------------- Initialisation of init_energy */
if (init_flag)
  {
  E_40+=0;IdT+=0;IT+=0;g+=0; T_inside+=0; /* dummy */
  /* parameter-check */
  if (mc<=0) {printf("mc of unit:%d <=0 ",unit);exit(-1);}
  for (i=1;i<=n;i++)
    {
    init_energy[unit][i]=deriv_o[unit][1]*mc;
    if (i>1) deriv_n[unit][i]=deriv_o[unit][i]=deriv_o[unit][1];
    if (a) printf("T_init[%d]:%.1f energy:%.4f \n"
           ,i,deriv_o[unit][1],init_energy[unit][1]);
    }
  };

/* old one:  rad_processor(tsim,day_of_year,Ibh,Idh,0.2,E_40,slope,orientation
                    ,rad_proc,lenght,latitude); */
rad_processor(tsim,day_of_year,Ibh,Idh,albedo,0 /*E_40*/
             ,slope ,orientation,rad_proc,lenght,latitude,Rd,Rr);

IgT=rad_proc[7];
Qsol=eps_w*A*IgT*h;

/* get heatflow from node to node */
Qi[1][1]= -Qrad-Qconvi;
for (i=2;i<=n;i++) Qi[i][1]=kA*(deriv_o[unit][i-1]-deriv_o[unit][i]);
Qi[n+1][1]= -1*(Qrad_outside+Qconva+Qsol);

/* get capacity energy from each node */
for (i=1;i<=n+1;i++) qp_sum[unit][i][1]+=Qi[i][1];
for (i=1;i<=n;i++) 
  {                                  
  energy[i]= init_energy[unit][i]
                 +qp_sum[unit][i  ][1] /* this one to the last node  */
                 -qp_sum[unit][i+1][1]; /* this one to the next node i+1 */
  deriv_n[unit][i]=energy[i]/mc; 
  if (a) printf("Qi[%d][1]:%.4f  T[%d]:%.4f energy[i]:%.1f \n"
     ,i,Qi[i][1],i,deriv_n[unit][i],energy[i]); 
  }


if (a) printf("mc:%.2f Tsky:%.4f Qrad_outside:%.4f "
              "Qconva1:%.4f Qsol1:%.4f Qconvi:%.4f  Qrad:%.4f \n"
              ,mc,Tsky,Qrad_outside,Qconva,Qsol,Qconvi,Qrad); 

 /*-------------- set outputs -----------------------------*/

out_n[unit][1]=deriv_n[unit][1];  /*T_inside surface */
out_n[unit][2]=0;           	  /* Alpha convection to inside */
out_n[unit][3]=eps;  			  /* epsilon inside surface */
out_n[unit][4]=slope;      		  /* angle between horizontal and wall */
out_n[unit][5]=orientation; 	  /* orientation 0=south -45=east 45=west */

out_n[unit][6]=Qsol; /* system gains [Ws]*/
out_n[unit][7]=-Qrad_outside-Qconva ;/* system losses */
   /* total internal energy contents due to 0  Celsius */
out_n[unit][8]=0;			
 for (i=1;i<=n;i++) out_n[unit][8]+=energy[i] ; /* (deriv_n[unit][i]*mc);	*/
out_n[unit][9] = deriv_n[unit][1];
out_n[unit][10]= deriv_n[unit][n];
out_n[unit][11]=0; /* 1/((1/alpha_convi)+(par[unit][2]/s)+(1/alpha_conva)); */
out_n[unit][12]=IgT;
return (1);
}
/*-------------------------------------------------------------------*/

