/* weather.interpolate.c */
/*       10        20        30        40        50        60        70       */
/*3456789012345678901234567890123456789012345678901234567890123456789012345671*/
/*----------------------------------------------------------------------------*/

#include "global.h"
#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <errno.h>
#include "rad_processor.h"

#define END -999.99

/*
000614 weather_read changed to weather() to get same object and routine name
000615 update original weather.interpolate.c with current weather.c       
001107 current_time_entry=next_time_entry.. if i_line++ 
001109 weather_init call excluded from init-segment , 2nd day re-init
010304 weather_init on init_flag() init_flag=0; later
       pedantic prints par7
010309 exit on count>WEATHER_SIZE
010627 bug current time sync: write out on tsim>=next_time_entry
       timestep h added to args of weather and weather_init
010702 uncommented rewritten nr_of_columns from par7 activated again
       if interpolate flag set , input_initial can be used to omit interpol
       see 'omit_interpolate'
       weather_init: while ( (strlen(str[++k])>=1) && (!ready) )
010710 stebus omit interp was buggy (no -3)
010920 chriwi: par1 timezone added
010923 include "rad_processor.h"
011106 par8 defined: weighting factors from input initial 
       check omit_interpolate operation!
011211 else weight_factor=1 added
020503 comments update
*/


double weather_init(int unit,double h,double **out_o,double **out_n,
                    int *nr_of_columns,double *weather,int year_start,int month,
                    int day,int unix_flag,int *weather_flag,double **par);



/*       10        20        30        40        50        60        70       */
/*3456789012345678901234567890123456789012345678901234567890123456789012345671*/
/*----------------------------------------------------------------------------*/


/*--------------WEATHER_READER---TYPE 9 --------------------------------------*/

int weather(info,radiation,tsim,h,unit,par,out_o,out_n,year_start,
            month,day,day_flag)
             struct sim_struct *info; /* see global.h */
             struct radiation_struct *radiation; /* see rad_processor.h */
             double tsim;    /* simulation time in seconds of this day */
             double h;       /* sim time step */
             int     unit;    /* current unit number */
             double **par;    /* parameter array par[unit][1..PARMAX] */
             double **out_o;  /* output array with privious state of 
                                 out[unit][1..n] */
             double **out_n;  /* output array with current state */
             int year_start;  /* filename generate */
             int month;
             int day;
             int day_flag;     /* if tsim==0 --> weather_init */
{
/*  etc/xfig.awk

    TYPE 09
    PRIORITY  1
    HYDRAULIC 99


    PAR1: 1 [h] timezone in hours due to greenwich time (+1h=1h east)
    PAR2: 600 [s] data timestep
    PAR3: -7.85 [degrees] geogr lenght: east (from greenwich) is negative 
    PAR4: 48 [degrees]    latitude
    PAR5: 13 [] data base, 1: use ./weather/try01/
                           if 01: from ./weather/try01/??????.dat
                           if 02: from ./weather/try02
                             ...
                           if 12: try12
                           if 13: user defined weather

    PAR6: 0 [] 1:interpolate,  0:non_interpolated readings from 'weather.dat'
    PAR7: 0 [1] 1:pedantic prints
    PAR8: 0 [1] 0:nothing else 1:use weighting factors from input initial 

    OUT1: [1] 'col1' column 1, column 0 is time
    OUT2: [1] 'col2' column 2
    OUT3: [1] 'col3' column 3
    OUT4: [1] 'col4' column 4
    OUT5: [1] 'col5' column 5
    OUT6: [Ws] 'sys_in'  reserved for energy balance
    OUT7: [Ws] 'sys_out' reserved for energy balance
    OUT8: [Ws] 'Q_int'   reserved for energy balance
    OUT9: [1] 'col6' column 6  (put column 6 to out 9) 
    OUT10: [1] 'col7' column 7
    OUT11: [1] 'col8' column 8
    OUT12: [1] 'col9' column 9
    OUT13: [1] 'col10' column 10
    OUT14: [1] 'col11' column 11
    OUT15: [1] 'col12' column 12
    OUT16: [1] 'col13' column 13
    OUT17: [1] 'col14' column 14
    OUT18: [1] 'col15' column 15
    OUT19: [1] 'col16' column 16
    OUT20: [1] 'col17' column 17

    INP1: 0 'col1' column 1 (out 1) 0:default, 1:if interpolate omit interpol
    INP2: 0 'col2' column 2 (out 2)
    INP3: 0 'col3' column 3 (out 3)
    INP4: 0 'col4' column 4 (out 4)
    INP5: 0 'col5' column 5 (out 5)
    INP6: 0 'col6' column 6 (out 9)
    INP7: 0 'col7' column 7 (out 10)
    INP8: 0 'col8' column 8 (out 11)
    INP9: 0 'col9' column 9 (out 12)
    INP10: 0 'col0' column 10 (out 13)
    INP11: 0 'col1' column 11 (out 14)
    INP12: 0 'col2' column 12 (out 15)
    INP13: 0 'col3' column 13 (out 16)
    INP14: 0 'col4' column 14 (out 17)
    INP15: 0 'col5' column 15 (out 18)
    INP16: 0 'col6' column 16 (out 19)
    INP17: 0 'col7' column 17 (out 20)
 */

int a=0;                     /*-----------------------ANALYSE a=1---------*/
const int b=0;                       /*---debug flag second level--------*/
static int line=0;
static int init_flag=1;
static double *weather;
static double current_time_entry=0; 
static double next_time_entry=0; 
static double timeIntervallCurrent=0;
static int interpolate_flag=0;
static int     weather_flag[1]; /*set by weather_init,reset by weather_reader */
static double time_max_of_database;

double timeDueToMessurement=0; /* from -1800 to 1800 sec for 1h database */
double deltaValue=0; /* Value_next-Value_current or Value_current-Value_last */
double m=0; /* steepness for interpolation */
static int nr_of_columns; /* including TIME-row !! */
int unix_flag=1;
int i;
int pedantic=par[unit][7];
int omit_interpolate;
double weight_factor=1.0;
int use_weight_factor=(int) PAR (8);
                                                            
if (init_flag) {
 /* init_flag=0; ??? */
 radiation->timezone=par[unit][1];
 radiation->length  =par[unit][3]; /*lenght due to solar noon */
 radiation->latitude=par[unit][4]; /* latitude */
 if (radiation->length==0) 
       printf("main.c: geograph lenght Par3 weather.c is 0 !!!!!!!\n");
 if (radiation->latitude==0)  printf("main.c: geograph latitude is 0 !!!!\n");
 weather=(double*)
 malloc(sizeof(double)*(OUT_MAX0+1)*WEATHER_SIZE);
 if (weather==NULL)
   { printf("malloc ERROR weather-array\n");exit (-1);}
 weather_flag[0]=1;

 timeIntervallCurrent=par[unit][2];
 interpolate_flag=(int) par[unit][6];
 }; /* end init */
if ( (day_flag) || (init_flag) ) time_max_of_database=
              weather_init(unit,h,out_o,
              out_n,&nr_of_columns,weather,year_start,month,
              day,unix_flag,weather_flag,par);
init_flag=0;
if (a) printf("weather: nr_of_columns:%d \n",nr_of_columns );

if (weather_flag[0]) {
  line=0;
  next_time_entry=weather[line*nr_of_columns+0];
  weather_flag[0]=0; /* reset flag */
}


current_time_entry=weather[(line+0)*nr_of_columns+0];
if (weather[(line+1)*nr_of_columns+0]==END) /* check end of database */
     next_time_entry=current_time_entry+timeIntervallCurrent;
else next_time_entry=weather[(line+1)*nr_of_columns+0];
if ((    (next_time_entry-current_time_entry)!=timeIntervallCurrent) 
      && (pedantic)
   )
   printf(" changing timeIntervalls in database tsim:%.4f sec !! "
          "t1:%.1f t2:%.1f\n",tsim,current_time_entry,next_time_entry);
timeIntervallCurrent=next_time_entry-current_time_entry;
if ( next_time_entry+h<tsim) {
 printf("weather:date:%2d:%2d:%2d next_time_entry:%.1f+%.1f < tsim:%.1f \n"
        " current_time_entry:%.1f \n",
       year_start,month,day,next_time_entry,h,tsim,current_time_entry); 
 /* exit (-1); */
}

if ( (interpolate_flag) || (tsim>=next_time_entry) ) {
   /* check next weather_dat_time ! , synchronize tsim with weather-data */
   while (   (weather[(line+1)*nr_of_columns+0]<=tsim)
          && (weather[(line+1)*nr_of_columns+0]!=END)
         )
     { 
     line++;
     current_time_entry=next_time_entry=weather[line*nr_of_columns+0]; 
     if (b) printf("while-loop line:%d nex_time_entry:%.2f\n"
            ,line,next_time_entry);
     }

   timeDueToMessurement=tsim-current_time_entry-0.5*timeIntervallCurrent;

   for (i=0;i<=nr_of_columns-1;i++) /* 0:time 1..nr_of_columns */
     { /* take care: out 6,7,8 are 0 !! ( sys_energy)  */
       if ( (line==0) && (timeDueToMessurement<0) ) deltaValue=0;
       else /* there should be a copy of data-line in 'END' line of weather[] */
        {
        if (timeDueToMessurement<0) deltaValue=weather[(line-0)*nr_of_columns+i]
                                             -weather[(line-1)*nr_of_columns+i];
        else /* second timeintervall */
         {
         if (tsim<time_max_of_database+0.5*timeIntervallCurrent) 
                                 deltaValue= weather[(line+1)*nr_of_columns+i]
                                            -weather[(line-0)*nr_of_columns+i];
         else deltaValue=0;
         }
        };

      if (timeIntervallCurrent>0) m=deltaValue/timeIntervallCurrent;
      else                        m=0;
      

      /* Q&D hack chriwi 010702: prevent interpolating on input_initial==1 */
      omit_interpolate=( (int) out_n[unit][i+OUT_MAX0])==1; /* is that 1 ?*/

      if ( use_weight_factor ) { /* use initial values to multiply outs */
        if (    ( ( out_n[unit][i+OUT_MAX0])!=0) 
             && ( ( out_n[unit][i+OUT_MAX0])!=1) 
           ) weight_factor=out_n[unit][i+OUT_MAX0];
        else weight_factor=1; /* set following factors */
      };    
          

      if ( (interpolate_flag) && !omit_interpolate ) 
       { 
       if (i<=5) out_n[unit][i  ]=out_o[unit][i  ]=weight_factor*
          (weather[(line-0)*nr_of_columns+i]+m*timeDueToMessurement);
       else      out_n[unit][i+3]=out_o[unit][i+3]=weight_factor*
          (weather[(line-0)*nr_of_columns+i]+m*timeDueToMessurement);
       }
      else /* no interpolation */
       { 
       if (i<=5) out_n[unit][i  ]=out_o[unit][i  ]=
          weight_factor*weather[(line-0)*nr_of_columns+i]; 
       else      out_n[unit][i+3]=out_o[unit][i+3]=
          weight_factor*weather[(line-0)*nr_of_columns+i];
       }
     }; /* i=0;i<=nr_of_columns-1 */

   if (a)
     {
     printf("tsim:%.1f current_time_entry:%.1f next_time_entry:%.1f \n"
           ,tsim,current_time_entry,next_time_entry);
     printf("\nout_weather: tsim:%.1f t_read:%.1f line:%d "
            "timeIntervallCurrent:%.1f \n out_n:"
             ,tsim,weather[(line  )*nr_of_columns],line , timeIntervallCurrent);
     for (i=0;i<=nr_of_columns+3;i++) printf("%.2f ",out_n[unit][i]);
     printf("\n");
     }
} /* if ( (interpolate_flag) || (tsim>next_time_entry) ) */
 


if ( tsim>time_max_of_database+timeIntervallCurrent )  /* exit,
                                                        if tsim >tim_max +dT */
  printf("no following data in 'weather.dat' "
         "tsim:%.2f  time_max_of_database:%.2f timeIntervallCurrent:%.2f\n"
         , tsim,time_max_of_database ,timeIntervallCurrent); 

return (1);
} /* weather() */







/* --------------------------------WEATHER_INIT-------------------------------*/

double weather_init(int unit,double h,double **out_o,double **out_n,
                    int *nr_of_columns,double *weather,int year_start,int month,
                    int day,int unix_flag,int *weather_flag,double **par)
{
const int a=0;  /*---debug flag --------*/
static FILE *pos;
#define LETTER_PER_LINE 1000
#define WORDS_MAX       50
char Zeile[LETTER_PER_LINE];
char *rest;
char str[WORDS_MAX+1][80];                    /* words in this line */
char file_name[]="weather/try00/940101.dat";
int date[4];
int count;                           /* current line */
int k;                               /* internal counter*/
double time1,time_in_sec=0;          /*end_time,start_time */
int hour,min,sec;
int offset=6; /* add extra characters for 'try00/' */
double dT1;                          /* time intervall */
double time_max=0;
int ready=0;
int columns=0;

if (a) printf("year_start:%d , month:%d , day:%d\n"
              ,year_start,month,day);

if (par[unit][5]==0) /* default for weather-data base */
 {
 sprintf(&file_name[0],"weather/940101.dat");
 offset=0;
 }
else
 {
 sprintf(&file_name[11],"%d",(int) par[unit][5]/10);
 sprintf(&file_name[12],"%d",(int) par[unit][5]-10*(int) (par[unit][5]/10) );
 if (a) printf("data base:%d%d\n",(int) par[unit][5]/10
                                 ,(int) par[unit][5]
                              -10*(int) (par[unit][5]/10) );
 sprintf(&file_name[13],"/940101.dat");
 }

sprintf(&file_name[8+offset],"%d",(int) year_start/10);
sprintf(&file_name[9+offset],"%d",(int) ( year_start-10*(int) (year_start/10) ));

date[0]=(int) month/10;  /*2nd position in month */
date[1]=month-date[0]*10;
date[2]=(int) day/10;    /*2nd position in day */
date[3]=day-date[2]*10;

/*-------------------------------generate input-file-name -*/
for (k=0;k<=3;k++)  /*k is position in string */
 sprintf(&file_name[10+offset+k],"%d",date[k]);
sprintf(&file_name[10+offset+4],".dat"); /*extension ends with \0 */
if ( !unix_flag ) file_name[7+offset]='\\'; /* attention in C */

if ((pos=fopen(file_name,"r"))==NULL)
   { printf("datafile '%s' not found \n",file_name);
     printf("errno:%d \n",errno);  exit(-1); }
if (a) printf("datafile '%s' found \n",file_name);
/*------------------READ HEADLINE----------------------*/
for (k=0;k<=WORDS_MAX;k++) str [k][0]='\0'; /* string-end */
if (fgets(Zeile,LETTER_PER_LINE,pos)==NULL) { 
   printf("no weather-data \n");exit (-1);}
if (sscanf(Zeile,"%s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s "
                 "%s %s %s %s %s %s %s %s %s %s %s"
                   ,str [0] ,str [1] ,str [2] ,str [3] ,str [4] ,str [5]
                   ,str [6] ,str [7] ,str [8] ,str [9] ,str [10],str [11]
                   ,str [12],str [13],str [14],str [15],str [16],str [17]
                   ,str [18],str [19],str [20],str [21],str [22],str [23]
                   ,str [24],str [25],str [26],str [27],str [28],str [29]
                   ,str [30]
           )==EOF)
  { printf(" line non existent \n") ; };
if (a) for (k=0;k<=WORDS_MAX;k++) printf("Label%d:%s\n",k,str [k]);

/*-------------------Print Headline---, get columns---------------------------*/
if (1) {
 printf("%s: ",file_name);
 k=0;ready=0;
 do { /* run k thru headline entries 0:time n:last_label */
  printf("%d:%s  ",k,str[k]);
  if (strlen(str[k+1])<1) { 
     *nr_of_columns=k+1; /* write back to weather() including time_row */
     columns=k; /* index (columns+1)*count+i */
     ready=1;
  } /* if (strlen(str[k])<1) */
  k++;
 } while ( !ready ); /* ++k<=columns)&&(!ready) );*/
 printf("\n");
} /* if (1) */
if ( (columns<2) || (columns>OUT_MAX0-1) )
{ printf("weather_init:  columns:%d <2 or >OUT_MAX0-1 \n",columns);exit(-1);};

fgets(Zeile,LETTER_PER_LINE,pos); /* empty line */
if (  (strlen(Zeile)>2)
    &&(Zeile[0]!='#') ) { printf("2nd line should be empty:%s.end\n",Zeile);exit (-1);  }

count=0;

/* read all lines */
while ( (fgets(Zeile,LETTER_PER_LINE,pos) != NULL) && (count<=WEATHER_SIZE) )
     {
       if (sscanf(Zeile,"%s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s %s "
                        "%s %s %s %s"
                        "%s %s %s %s %s %s %s %s %s %s"
                  ,str [0] ,str [1] ,str [2] ,str [3] ,str [4] ,str [5]
                  ,str [6] ,str [7] ,str [8] ,str [9] ,str [10],str [11]
                  ,str [12],str [13],str [14],str [15],str [16],str [17]
                  ,str [18],str [19],str [20],str [21],str [22],str [23]
                  ,str [24],str [25],str [26],str [27],str [28],str [29],
                   str [30]
                  )
          ==EOF)
         { printf(" error in reading data-line \n");};
       if (strlen(Zeile)>3)
         {
           time1=strtod(str[0],&rest); /* time1=hour/min/sec */
           /* time-column from hour/min/sec --> time_in_sec */
           hour=(int) ( (time1-1e+6*(int) (time1/1e+6))/10000);
           min =(int) ( (time1-1e+4*(int) (time1/1e+4)) /100 );
           sec =(int) ( (time1-1e+2*(int) (time1/1e+2)) /1   );

           weather[(columns+1)*count+0]=hour*3600+min*60+sec; /*abs. sec's */
           for (k=1;k<=columns;k++)
             weather[(columns+1)*count+k]=strtod(str[k],&rest);

           if (a) {
                  printf("weather_init: line:%d index:%d\n"
                         ,count,(columns+1)*count+0);
                  for (k=0;k<=columns;k++)
                           printf("%d:%.1f ",k,weather[(columns+1)*count+k] );
                                   printf("\n");
                  };
         };

       if (count==0)    /*Initial Values*/
         {
           /*time=weather[0];*/
           time_in_sec=hour*3600+min*60+sec;
           out_n[unit][0]=out_o[unit][0]=time_in_sec;
           for (k=1;k<=columns;k++)
              {
                /* save out 6,7,8 for sys_energies */
                if (k <= 5)
                  out_n[unit][k]=out_o[unit][k]=weather[k];
                else
                  out_n[unit][k+3]=out_o[unit][k+3]=weather[k];
              }
           if (a)
           printf("START: time=%2d:%2d:%2d \n",hour,min,sec);
         };
       if (count==1) dT1=hour*3600+min*60+sec-time_in_sec;
       if ( (count>=2) && ((fabs(hour*3600+min*60+sec-time_in_sec-dT1))>610 ))
         {
           if ((strlen(Zeile)<=3))
             printf("empty line time:%d:%d:%d \n",hour,min,sec);
           else
             printf("space in Database time:%d:%d:%d \n",hour,min,sec);
         };
       if (count >= 1) time_max=weather[(columns+1)*(count-1)+0];
        else time_max=0;
       time_in_sec=weather[(columns+1)*count+0];
       count++;
     }; /* while run through all data lines */
weather[(columns+1)*count+0]=END; /* set end of context */
if (time_in_sec<1) printf("check time-column in weather.dat (empty line)!!\n");
else
  time_max=time_in_sec;

if (a)
{
printf("STOP:  time=%2d:%2d:%2d \n",hour,min,sec);
printf("weather.dat includes %d lines in time_intervall:%.2f [h]  "
       "with timestep:%.0f sec\n",(count-1),time_max/3600,dT1);
}

if (count>WEATHER_SIZE) { printf("\n\n'weather.dat' out of range !!!\n\n\n");
                          printf("WEATHER_SIZE in global.h !! \n");
                          exit (-1); }
fclose(pos);
if (a) printf("time_max:%.2f sec\n",time_max);
weather_flag[0]=1;

return (time_max);  /* return time-step  */
}
 


