/* zone.c */
/*       10        20        30        40        50        60        70       */
/*3456789012345678901234567890123456789012345678901234567890123456789012345671*/
/*----------------------------------------------------------------------------*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include "global.h"
#include "rad_processor.h"

/*
000607 pointer lenght,latitude
020321 update labels etc
020404 out8: -Q12-Q13; using out_n at conv and rad gains
*/
/*------------------ZONE TYPE 20---------------------------*/
int zone(unit,par,in_u,in_nr,out_n,out_o,deriv_o,deriv_n,h,init_energy,
         qp_sum,tsim,init_flag,day_of_year,lenght,latitude,type)
           int unit;   /* current unit number */
           double **par;  /* parameter array par[unit][1..PARMAX] */
           int **in_u;    /* input unit array, input connections  */
           int **in_nr;   /* input number array     */
           double **out_o; /* output array with privious state of  */
           double **out_n; /* output array with current state */
           double **deriv_o; /* derivatives: used for diffential equations */
           double **deriv_n; /*  new state of deriv */
           double h;         /* sim time step */
           double **init_energy; /* initial energy in RC-componentes [Ws] */
           double ***qp_sum; /* energy to one node qp[unit][n][m] */
           double tsim;      /* simulation time in [s] of this day */
           int init_flag;    /* set at initial loop of sim */
           int day_of_year;  /* 1..365 */
           double *lenght;       /* geographic lenght for rad_processor */
           double *latitude;     /* geographic latitude */
           int *type;        /* type_number */       
{
/*   ./etc/xfig.awk      
  
     TYPE 20
     PRIORITY  12         used by config routine
     HYDRAULIC 99         used by config routine
  
     PAR1: 10000 [kg] 'mc' iof walls total kg Water
     PAR2: 4 [m^2] 'Awin' total area of windows
     PAR3: 50 [m^3] 'VolAir' Volume of air
     PAR4: 0.8 [1] 'absorp' absorption short wave,not linked yet
     PAR5: 0.6 [1] 'emiss' emmisivity long wave,linked to out3
     PAR6: 40 [W/K] 'kA_Wall' heat_loss to ambient kA_walls
     PAR7: 0.7 [1] 'g' g_Value total energy through window
     PAR8: 7.7 [W/m^2K] 'conv' convective heattransfer coefficient
                       h_inside:  DIN 7.7   W/m^K 
                       h_outside: DIN 23.0  W/m^K 
     PAR9: 60 [m^2] 'AinsideWall' area of inside walls (convection to air-node)
     PAR10: 2 [W/m^2K] 'k_win' k-Value for windows
     PAR11: 25 [C] 'Tclose' temperature to close roller blinds (set g to 0.1)
     PAR12: 0.0 [W/K] 'kA_bottom' 
     PAR13: 45 [1] 'slope' slope , not used yet
     PAR14: 0 [] 'orien'  (0:south,90:west,-90:east)
     PAR15: 4 [m^2] 'Awin' area window2
     PAR16: 0.7 [1] 'g_win2' g_Value window2
     PAR17: 2 [W/m^2K] 'k_win2' k-Value window2
     PAR18: 45 [] 'slope_win2' slope window2
     PAR19: 0 [1] 'orien2' orientation window2
     PAR20: 4 [m^2] 'Ati' area transpar insolation ti
     PAR21: 0.7 [1] 'g_ti' g_Value ti
     PAR22: 2 [W/m^2K] 'k_ti' k-Value ti
     PAR23: 0 [] 'orien_ti' orientation ti
     PAR24: 1 [W] 'Pint' 1: Pintern coupled to TWall 0:to Tair
     

     INP1: 0 [Ws] deltaQ_convective_inlet heating
     INP2: 0 [Ws] deltaQ_radiative_inlet heating
     INP3: 0 [W] solar radiation in zone, E_90
     INP4: 20 [C] ambient temperature
     INP5: 0.2 [1/h] 'infil' infiltration par3/hour
     INP6: 0.0 [kW] internal load 
     INP7: 10.0 [C] Temp ground 
     INP8: 10.0 [C] Temp active infiltration
     INP9: 0  [W] solar Iglobal horizontal Igh
     INP10: 0 [W] solar Idiff horizontal Ibd

     OUT1: [C] 'Tconv' temperatur ambient for convective exchange T_Air
     OUT2: [C] 'Trad' temperatur ambient for radiative exchange T_Walls
     OUT3: [] 'emiss' emissivity ??
     OUT4: [W] 'Ploss' ambient: kA and infiltration
     OUT5: [W] 'Pinfil' loss infiltration
     
     OUT6: [Ws] 'sysgain' system inlet energy = eta*E
     OUT7: [Ws] 'sysloss' system outlet energy =k*dT
     OUT8: [Ws] 'sysenerg' internal energy 

     OUT9: [W] 'P_sun' P_passiv sun
     OUT10: [Ws] 'QairLoss' deltaQ air outlet (infiltration&activ convection)
     OUT11: [kg/s] 'mp_air'   air outlet (from inp5)
     OUT12: [W] 'Pgain' internal gains/load

     DERIV1: 20 [C] 'TAir' T_Air
     DERIV2: 18 [C] 'Twall' T_Walls
*/

int a=0; /*-------------------------ANALYSE a=1----------*/
                    
double mc_walls =PAR(1) *4190;
double k_window =PAR(2); /* W/m2K */
double Vol_Air  =PAR(3); /* m3 */

double A_w2    =par[unit][15]; /* decleration window 2 */
double slope_w2=par[unit][18];
double orien_w2=par[unit][19];
double kA_w2   =par[unit][17]*par[unit][15];
double g_w2    =par[unit][16];
  
double A_ti    =par[unit][20];
double g_ti    =par[unit][21];
double kA_ti   =par[unit][22]*par[unit][20];
double orien_ti=par[unit][23];

double infil=out_o IN(5); /* infoltration */
double Ib_hor=out_n IN(9)-out_n IN(10); /* Ihb=Ihg-Ihd beam radiation */
double Id_hor=out_n IN(10); 

double mc_air,mp_air,kA_window,kA_walls,kA_bottom;
double Q11,Q12,Q13,Q14,Q15,Q16,Q17,Q18;
double energy_walls,energy_air;
double cp_air=1.005; /*cp_air=1.005 kJ/kgK */
double rho_air=1.2; /* VDI 20C kg/m^3 */
double rad_proc[13]; /* array for radiation processor */
double IgT_w2=0;               /* global tilted */
double IgT_ti=0;

/*cp_air=1.005 kJ/kgK rho=1.29kg/m^3 if T_amb=20^oC */
mc_air=Vol_Air*cp_air*rho_air*1e+3;
mp_air=Vol_Air*rho_air*infil /(3600*h);

kA_window=k_window*par[unit][10];
kA_walls =par[unit][6];
kA_bottom=par[unit][12];

/* Initialisation of init_energy */
if (init_flag) {
  if (    (mc_air  <=0) || (mc_walls<=0) ) {
     printf("exit in ZONE, caused bei heatcapacity\n"); exit (-1);
  }
  if (in_u [unit][8]==unit) { in_u [unit][8]=in_u [unit][4];
                              in_nr[unit][8]=in_nr[unit][4];
                              printf("Tin_air set to ambient \n"); 
  }
      
  if (deriv_o[unit][1]<=0) deriv_o[unit][1]=deriv_n[unit][1]=20;

  if (deriv_o[unit][2]<=0) deriv_o[unit][2]=deriv_n[unit][2]=20;

  /*T_Air */ /*cp_air=1.005 kJ/kgK rho=1.2kg/m^3 if T_amb=20^oC */
  init_energy[unit][1]=deriv_o[unit][1]*Vol_Air*1005*rho_air;
      /* T_Walls */
  init_energy[unit][2]=deriv_o[unit][2]*mc_walls;
  if (a) printf("energy_Air:%.4f  energy_walls:%.4f \n",
                init_energy[unit][1],init_energy[unit][2]);
}; /* init_flag */

  rad_processor(tsim,day_of_year,Ib_hor,Id_hor,0.2,0,slope_w2,orien_w2
                ,rad_proc,lenght,latitude,0 /*Rd ext*/,0 /*Rr*/);
  IgT_w2=rad_proc[7]; /* window 2 */
  rad_processor(tsim,day_of_year,Ib_hor,Id_hor,0.2,0,90      ,orien_ti
                ,rad_proc,lenght,latitude,0 /*Rd ext*/,0 /*Rr*/);
  IgT_ti=rad_proc[7]; /* transparent insulation */

  Q11=  ( deriv_o[unit][1] -
          out_o[  in_u[unit][4] ]     /* ambient losses from window */
               [ in_nr[unit][4] ] )*(kA_window+kA_w2)*h
       +(deriv_o[unit][1] -out_o[  in_u[unit][8] ]
                                [ in_nr[unit][8] ])*mc_air*h*out_o[  in_u[unit][5] ]  
           /* infiltration factor */                              [ in_nr[unit][5] ]/3600 ;

  /* Q_convective heating system */
  Q12=out_n IN(1);    /* Q_convective */

  /* Q_radiative heating system */
  Q13=out_n IN(2); /* Q radiative  */

  /*Q_sun: inc radiation*A*g */
  if (deriv_o[unit][1]>par[unit][11]) /* if T_air>Tmax close roller blind */
    Q14=out_o[  in_u[unit][3] ]
             [ in_nr[unit][3] ]*par[unit][2]*0.0*h   /* g value set to 0.0 */
                                +IgT_w2*A_w2*0.0*h
                                    +IgT_ti*A_ti*0.0*h;
  else       
   Q14=out_o[  in_u[unit][3] ]      
            [ in_nr[unit][3] ]*par[unit][2]*par[unit][7]*h /* use g value par7 */
                              +IgT_w2*A_w2*g_w2*h
                          +IgT_ti*A_ti*g_ti*h;

  /* Q_convective air-walls */
  Q15=par[unit][9]*par[unit][8]*h
      *(deriv_o[unit][2]-deriv_o[unit][1]);

  /* Q_loss ambient walls  & ti losses  */
  Q16=( deriv_o[unit][2] -
          out_o[  in_u[unit][4] ]     /*ambient temperature */
                   [ in_nr[unit][4] ] )*(kA_walls+kA_ti)*h;
        
  Q17=( deriv_o[unit][2] -
         out_o[  in_u[unit][7] ]     /* ambient losses from walls to bottom */
                  [ in_nr[unit][7] ] )*kA_bottom*h;
          
  Q18=out_o[  in_u[unit][6] ]        /* internal loads */
           [ in_nr[unit][6] ]  *h*1e3;
  

  if (a) printf("Q11:%.4f\nQ12:%.4f\nQ13:%.4f\nQ14:%.4f\n"
                "Q15:%.4f\nQ16:%.4f\n"
                ,Q11,Q12,Q13,Q14,Q15,Q16);

  /*  qp_sum[unit][deriv][path] */
  qp_sum[unit][1][1]+=Q11; /* ambient losses from window */
  qp_sum[unit][1][2]+=Q12; /* Q_convective */
  qp_sum[unit][1][3]+=Q13; /* Q radiative  */
  qp_sum[unit][1][4]+=Q14; /* g value set */
  qp_sum[unit][1][5]+=Q15; /* Q_convective air-walls */
  qp_sum[unit][1][6]+=Q16; /* Q_loss ambient walls  & infiltration */
  qp_sum[unit][1][7]+=Q17; /* Q loss bottom */
  qp_sum[unit][1][8]+=Q18; /* internal gains */

  energy_air= init_energy[unit][1]
               -qp_sum[unit][1][1]   /* Q_loss ambient */ 
               +qp_sum[unit][1][2]   /* Q_convective heating system */ 
               +qp_sum[unit][1][5]   /* Q_convective air-walls */
               +qp_sum[unit][1][8]*(1-par[unit][24]) ;  /* internal gains * flag */

  deriv_n[unit][1]=energy_air/mc_air;   /* temp_air */

  energy_walls= init_energy[unit][2]
                 +qp_sum[unit][1][3]  /* Q radiative heating system */ 
                 +qp_sum[unit][1][4]  /* Q_sun */
                 -qp_sum[unit][1][5]  /* Q_convective air-walls */
                 -qp_sum[unit][1][6]  /* Q_loss ambient walls */
                 -qp_sum[unit][1][7]  /* Q loss walls to bottom */
                 +qp_sum[unit][1][8]*par[unit][24] ;  /* internal gains * flag */

  deriv_n[unit][2]=energy_walls/mc_walls;   /* temp_walls */

  if (a) printf("T_Air:%.4f  T_Walls:%.4f \n"
                ,deriv_n[unit][1]
                ,deriv_n[unit][2]);

  /* energy-balance */

  /*--------------set outputs----------------------*/

  out_n[unit][1]=deriv_n[unit][1]; /* temp_air   */

  out_n[unit][2]=deriv_n[unit][2]; /* temp_walls */

  /* emissivity of walls = absorption longwave:=1*/
  out_n[unit][3]=par[unit][5]; /* emiss long wave */           

  /* Ploss ambient kA and infiltration  */
  out_n[unit][4]=(Q11+Q16+Q17)/h;

  /* Ploss infiltration */
  out_n[unit][5]=( deriv_o[unit][1] -
          out_o[  in_u[unit][4] ]     /*ambient temperature */
               [ in_nr[unit][4] ] )
        *( mc_air*out_o[  in_u[unit][5] ]  /* infiltration factor */
                       [ in_nr[unit][5] ]/3600 ); 

  /*System Energy and Power */
  out_n[unit][6]=Q14+Q18; /* System-Input-Energy [Ws]*/

  /* System-Output-Energy [Ws] */
  out_n[unit][7]=Q11+Q16+Q17; /* Q_loss    */

  /* internal energy [Ws]*/
  out_n[unit][8]=energy_air+energy_walls-Q12-Q13;  
  /* passiv P from sun */
  out_n[unit][9]=Q14/h;

  /* delta Q air */
  out_n[unit][10]=mp_air*cp_air*deriv_n[unit][1];
  /* mp air: [kg/s] mp   air outlet (from inp5) rho=1.29 */
  out_n[unit][11]=mp_air;
                       
  out_n[unit][12]=Q18/h;

  return 1;
}
/*---------------------------------------------------------------*/

