/* water_drop.c */
/*       10        20        30        40        50        60        70       */
/*3456789012345678901234567890123456789012345678901234567890123456789012345671*/
/*----------------------------------------------------------------------------*/

#include "global.h"
#include <stdio.h>
#include <stdlib.h>
#include <math.h>

/*---------------------------------------------------------------WATER_DROP---*/
int water_drop(unit,par,in_u,in_nr,out_o,out_n,deriv_o,deriv_n,h,init_energy,
               qp_sum,tsim,init_flag)
           int    unit;      /* current unit number */
           double **par;     /* parameter array par[unit][1..PARMAX]  */
           int    **in_u;    /* input unit array, input connections   */
           int    **in_nr;   /* input number array                    */
           double **out_o;   /* output array with privious state      */
           double **out_n;   /* output array with current state       */
           double **deriv_o; /* derivatives: used for diffential equ. */
           double **deriv_n; /*  new state of deriv                   */
           double h;         /* sim time step                         */
           double **init_energy; /* initial energy in RC-comp. [Ws]   */
           double ***qp_sum; /* energy to one node qp[unit][n][m]     */
           double tsim;      /* simulation time in [s] of this day    */
           int    init_flag; /* set at initial loop of sim            */

{
/* etc/xfig.awk
 
   TYPE 05
   PRIORITY  6          used by config routine
   HYDRAULIC 1          1-pole fluid  


   PAR1: 1 [] proportional part , not yet
   PAR2: 5 [kg] mc in kg water
   PAR3: 4.19 [kJ/kgK] (cp) not yet
   PAR4: 1 [W/K] kA

   INP1: 0 [Ws] deltaQ-inlet
   INP2: 0 [kg/s] mp-inlet
   INP3: 20.0 [C] ambient temperature

   OUT1: [Ws] deltaQ-outlet (losses)
   OUT2: [kg/s] mp-outlet set for pump circle
   OUT3: [C] T_re (real temperature)
   OUT4: []
   OUT5: []
   OUT6: [Ws] system inlet energy 
   OUT7: [Ws] system outlet energy 
   OUT8: [Ws] internal energy

   DERIV1: 10 [C] temp of load_place

*/

 int a=0; /*-------------ANALYSE a=1 ---------*/
 int pedantic=0; /* print stabilizing */

 double Q11,Q12,Q13,Q14;
 double mc=par[unit][2]*4190;
 double cp=par[unit][3]*1e3;
 double mp_cp_h=out_n[  in_u[unit][2] ]
                     [ in_nr[unit][2] ]*cp*h;
 double kA=par[unit][4];
 double kA_fluid=kA*10; /* kA to node */
 double energy=0;

 /* Initialisation of init_energy */
 if (kA_fluid==0) kA_fluid=10;
 if (init_flag)
   {
     if (mc<=0) {printf("mc in tap unit:%d!!\n",unit);exit(-1);}
     init_energy[unit][1]=deriv_o[unit][1]*mc;

      /*set default temp ambient */
      if (in_u[unit][3]==0) {
			     printf("inp 3 of tap u:%d set\n"
				    ,unit);
			     in_u [unit][3]=unit;
			     in_nr[unit][3]=33;
			     out_n[unit][33]=out_o[unit][33]=15;
			    }
   }
 /* node energy flow */

 Q11=out_n[  in_u[unit][1] ]       /* Q_in */
          [ in_nr[unit][1] ];
 /* no longer: Q12=-mp_cp_h*deriv_o[unit][1];    Q_WW_out */
 Q13=kA*(deriv_o[unit][1]-out_o[  in_u[unit][3] ]
                                [ in_nr[unit][3] ])*h;

 if (Q11>0)
     {
     double Tin,T_out;
     Tin=Q11/mp_cp_h;
     Q14=kA_fluid*(Tin-deriv_o[unit][1])*h;
     Q12=Q11-Q14; /* put losses */
     /*-------------proof numerical stability -----------------*/
     T_out=Q12/mp_cp_h;
     if (    (     ( Tin >deriv_o[unit][1])     /* if Qin has losses */
                && (T_out<deriv_o[unit][1])     /*  then it's wrong !! */
             )
          || (     (Tin  <deriv_o[unit][1])     /* if Qin gtes energy */
                && (T_out>deriv_o[unit][1])     /*  then it's wrong !! */
             )
        )
        { /* limit this temperature */
        T_out=deriv_o[unit][1];
        if (pedantic) printf("limit Tap temperature to :%.1f degrees \n",T_out);
        Q12=T_out*mp_cp_h;
        Q14=Q11-Q12;
        }
     if (a) printf("tap temp limited to :%.1f Tin:%.1f mp_cp_h:%.2f  \n"
             ,T_out,Tin,mp_cp_h);
     }
    else /* no flow, no problem ...  */
     Q14=Q12=0;




 if (    (mp_cp_h==0)  /* if their is inlet without mp */
      && (Q11  !=0)
    )
    {
    printf("TAP: unit:%d Qin!=0 for mp_cp_h==0 \n",unit); exit(-1);
    }

 /* write  qp_sum[unit][deriv][path] */
 qp_sum[unit][1][1]+=Q11;
 qp_sum[unit][1][2]+=Q12;
 qp_sum[unit][1][3]+=Q13;
 qp_sum[unit][1][4]+=Q14;

 energy= init_energy[unit][1]
             -qp_sum[unit][1][3]  /* Q_LOSS */
             +qp_sum[unit][1][4];  /* Q_WW_out */

 deriv_n[unit][1]=energy/mc;
 if ( (a) && (mp_cp_h>0) )
     printf("\ntap-dump--------:\n"
            "mp        :%8.4f kg/h \n"
            "Pnode     :%8.4f W \n"
	        " temp     :%8.4f \n"
            ,out_n[  in_u[unit][2] ]
                  [ in_nr[unit][2] ]*3600
            ,(Q11-Q12)/h
	        ,deriv_n[unit][1]
           );
/* if (mp_cp_h>0) printf("Pcool:%.2f \n",mp_cp_h*(Q12/mp_cp_h-10)/h);
 */

/*--------------set outputs----------------------*/
/*dQ_loss_water */
out_n[unit][1]=Q12; /* -Q13; if open end */
out_n[unit][2]=out_n[  in_u[unit][2] ]
                    [ in_nr[unit][2] ]; /* mp */

out_n[unit][3]=deriv_n[unit][1]; /* T_node */

 /* out_n[unit][4]=0;
    out_n[unit][5]=0;
 */
/*System Energy and Power */
out_n[unit][6]=0;       /* sys-in */
out_n[unit][7]=Q13;     /* sys out  */
out_n[unit][8]=energy;  /* sys cap */

return 1;

}


